"""
Relationship models for many-to-many entity linking
"""
from datetime import datetime

from sqlmodel import Column, Field, SQLModel, String


# Task relationship models
class TaskPerson(SQLModel, table=True):
    """Many-to-many relationship between tasks and people"""

    __tablename__ = "task_people"

    task_id: int = Field(foreign_key="tasks.id", primary_key=True)
    person_id: int = Field(foreign_key="people.id", primary_key=True)
    relationship_type: str = Field(default="mentioned", sa_column=Column(String(20)))
    strength_score: float = Field(default=1.0, ge=0.0, le=1.0)
    created_at: datetime = Field(default_factory=datetime.utcnow)

    class Config:
        from_attributes = True


class TaskProject(SQLModel, table=True):
    """Many-to-many relationship between tasks and projects"""

    __tablename__ = "task_projects"

    task_id: int = Field(foreign_key="tasks.id", primary_key=True)
    project_id: int = Field(foreign_key="projects.id", primary_key=True)
    relationship_type: str = Field(default="part_of", sa_column=Column(String(20)))
    strength_score: float = Field(default=1.0, ge=0.0, le=1.0)
    created_at: datetime = Field(default_factory=datetime.utcnow)

    class Config:
        from_attributes = True


class TaskTopic(SQLModel, table=True):
    """Many-to-many relationship between tasks and topics"""

    __tablename__ = "task_topics"

    task_id: int = Field(foreign_key="tasks.id", primary_key=True)
    topic_id: int = Field(foreign_key="topics.id", primary_key=True)
    relationship_type: str = Field(default="researches", sa_column=Column(String(20)))
    strength_score: float = Field(default=1.0, ge=0.0, le=1.0)
    created_at: datetime = Field(default_factory=datetime.utcnow)

    class Config:
        from_attributes = True


# Note relationship models
class NotePerson(SQLModel, table=True):
    """Many-to-many relationship between notes and people"""

    __tablename__ = "note_people"

    note_id: int = Field(foreign_key="notes.id", primary_key=True)
    person_id: int = Field(foreign_key="people.id", primary_key=True)
    mention_type: str = Field(default="mentioned", sa_column=Column(String(20)))
    mention_count: int = Field(default=1, gt=0)
    strength_score: float = Field(default=1.0, ge=0.0, le=1.0)
    created_at: datetime = Field(default_factory=datetime.utcnow)
    updated_at: datetime | None = None

    class Config:
        from_attributes = True


class NoteProject(SQLModel, table=True):
    """Many-to-many relationship between notes and projects"""

    __tablename__ = "note_projects"

    note_id: int = Field(foreign_key="notes.id", primary_key=True)
    project_id: int = Field(foreign_key="projects.id", primary_key=True)
    mention_type: str = Field(default="mentioned", sa_column=Column(String(20)))
    mention_count: int = Field(default=1, gt=0)
    strength_score: float = Field(default=1.0, ge=0.0, le=1.0)
    created_at: datetime = Field(default_factory=datetime.utcnow)
    updated_at: datetime | None = None

    class Config:
        from_attributes = True


class NoteTopic(SQLModel, table=True):
    """Many-to-many relationship between notes and topics"""

    __tablename__ = "note_topics"

    note_id: int = Field(foreign_key="notes.id", primary_key=True)
    topic_id: int = Field(foreign_key="topics.id", primary_key=True)
    mention_type: str = Field(default="discusses", sa_column=Column(String(20)))
    mention_count: int = Field(default=1, gt=0)
    strength_score: float = Field(default=1.0, ge=0.0, le=1.0)
    created_at: datetime = Field(default_factory=datetime.utcnow)
    updated_at: datetime | None = None

    class Config:
        from_attributes = True


# Topic relationship models
class TopicRelationship(SQLModel, table=True):
    """Two-way relationships between topics for knowledge graph"""

    __tablename__ = "topic_relationships"

    from_topic_id: int = Field(foreign_key="topics.id", primary_key=True)
    to_topic_id: int = Field(foreign_key="topics.id", primary_key=True)
    relationship_type: str = Field(sa_column=Column(String(30)))  # related_to, prerequisite_of, part_of, contradicts, extends
    strength_score: float = Field(default=1.0, ge=0.0, le=1.0)
    description: str | None = None
    created_at: datetime = Field(default_factory=datetime.utcnow)
    updated_at: datetime | None = None

    class Config:
        from_attributes = True


class TopicPerson(SQLModel, table=True):
    """Relationships between topics and people (experts, researchers)"""

    __tablename__ = "topic_people"

    topic_id: int = Field(foreign_key="topics.id", primary_key=True)
    person_id: int = Field(foreign_key="people.id", primary_key=True)
    relationship_type: str = Field(default="expert_in", sa_column=Column(String(20)))
    strength_score: float = Field(default=1.0, ge=0.0, le=1.0)
    created_at: datetime = Field(default_factory=datetime.utcnow)

    class Config:
        from_attributes = True


class TopicProject(SQLModel, table=True):
    """Relationships between topics and projects"""

    __tablename__ = "topic_projects"

    topic_id: int = Field(foreign_key="topics.id", primary_key=True)
    project_id: int = Field(foreign_key="projects.id", primary_key=True)
    relationship_type: str = Field(default="implements", sa_column=Column(String(20)))
    strength_score: float = Field(default=1.0, ge=0.0, le=1.0)
    created_at: datetime = Field(default_factory=datetime.utcnow)

    class Config:
        from_attributes = True



"""
Note model for immutable snapshots of processed file content
"""
from datetime import datetime
from typing import Any

from sqlmodel import JSON, Column, Field, SQLModel, String, Text


class Note(SQLModel, table=True):
    """Note table for immutable snapshots of processed file content"""

    __tablename__ = "notes"

    id: int | None = Field(default=None, primary_key=True)
    user_id: int = Field(foreign_key="users.id", index=True)
    source_file_id: int = Field(foreign_key="source_files.id")
    content: str = Field(sa_column=Column(Text))
    content_hash: str = Field(sa_column=Column(String(64), index=True))
    snapshot_at: datetime

    # Timestamps
    created_at: datetime = Field(default_factory=datetime.utcnow)

    # Processing metadata and flags
    processing_metadata: dict[str, Any] | None = Field(default=None, sa_column=Column(JSON))
    is_deleted: bool = Field(default=False)

    class Config:
        """SQLModel configuration"""
        from_attributes = True



"""
Source file model for tracking user's monitored files
"""
from datetime import datetime

from sqlmodel import Column, Field, SQLModel, String


class SourceFile(SQLModel, table=True):
    """Source file table for tracking user's monitored files"""

    __tablename__ = "source_files"

    id: int | None = Field(default=None, primary_key=True)
    user_id: int = Field(foreign_key="users.id", index=True)
    file_path: str = Field(sa_column=Column(String(1000)))
    relative_path: str = Field(sa_column=Column(String(1000)))
    file_hash: str = Field(sa_column=Column(String(64), index=True))
    file_size: int
    last_modified: datetime

    # Timestamps
    created_at: datetime = Field(default_factory=datetime.utcnow)
    updated_at: datetime | None = None

    # Status flags
    is_watched: bool = Field(default=True)
    is_deleted: bool = Field(default=False)

    class Config:
        """SQLModel configuration"""
        from_attributes = True



"""
Suggestion service for human-in-the-loop entity verification
"""
from typing import Any

from sqlalchemy import desc
from sqlmodel import Session, select

try:
    from fastapi import HTTPException, status
except ImportError:
    # FastAPI not available - CLI mode
    HTTPException = None
    status = None

from ..models.entity_learning import EntityLearning
from ..models.person import Person
from ..models.project import Project
from ..models.suggested_entity import SuggestedEntity
from ..models.topic import Topic
from .entity_recognition import EntityRecognitionEngine


class SuggestionService:
    """Service for managing entity suggestions and human verification"""

    def __init__(self, db: Session):
        self.db = db
        self.recognition_engine = EntityRecognitionEngine()

    def create_suggestions_from_text(self, text: str, note_id: int, user_id: int) -> list[SuggestedEntity]:
        """Process text and create entity suggestions"""
        result = self.recognition_engine.process_text(text)
        suggestions = []

        # Create suggestions for all entity types
        for entity_type, entities in result["entities"].items():
            for entity in entities:
                # Convert plural entity type to singular
                singular_type = self._pluralize_to_singular(entity_type)

                # Skip if we already have a suggestion for this entity in this note
                existing = self.get_suggestion_by_name_and_note(
                    entity.name, singular_type, note_id, user_id
                )
                if existing:
                    continue

                suggestion = SuggestedEntity(
                    user_id=user_id,
                    note_id=note_id,
                    entity_type=singular_type,
                    name=entity.name,
                    context=entity.context,
                    confidence=entity.confidence,
                    status="pending",
                    ai_metadata={
                        "original_text": entity.original_text,
                        "start_pos": entity.start_pos,
                        "end_pos": entity.end_pos,
                        "pattern_type": "regex" if entity.confidence > 0.8 else "natural_language"
                    }
                )

                self.db.add(suggestion)
                suggestions.append(suggestion)

        self.db.commit()
        return suggestions

    def get_pending_suggestions(self, user_id: int, limit: int | None = None) -> list[SuggestedEntity]:
        """Get all pending suggestions for a user"""
        statement = select(SuggestedEntity).where(
            SuggestedEntity.user_id == user_id,
            SuggestedEntity.status == "pending"
        ).order_by(desc(SuggestedEntity.confidence))  # type: ignore

        if limit:
            statement = statement.limit(limit)

        return list(self.db.exec(statement).all())

    def get_suggestions_by_note(self, note_id: int, user_id: int) -> list[SuggestedEntity]:
        """Get all suggestions for a specific note"""
        statement = select(SuggestedEntity).where(
            SuggestedEntity.user_id == user_id,
            SuggestedEntity.note_id == note_id
        ).order_by(desc(SuggestedEntity.confidence))  # type: ignore

        return list(self.db.exec(statement).all())

    def get_suggestion_by_name_and_note(self, name: str, entity_type: str, note_id: int, user_id: int) -> SuggestedEntity | None:
        """Check if suggestion already exists for this entity in this note"""
        statement = select(SuggestedEntity).where(
            SuggestedEntity.user_id == user_id,
            SuggestedEntity.note_id == note_id,
            SuggestedEntity.entity_type == entity_type,
            SuggestedEntity.name == name
        )

        return self.db.exec(statement).first()

    def confirm_suggestion(self, suggestion_id: int, user_id: int, create_new: bool = True, existing_entity_id: int | None = None) -> int | None:
        """Confirm a suggestion and optionally create the entity or link to existing"""
        suggestion = self._get_suggestion_by_id(suggestion_id, user_id)

        if create_new:
            entity_id = self._create_entity_from_suggestion(suggestion)
        elif existing_entity_id:
            # Link to existing entity
            entity_id = existing_entity_id
            # Add the suggested name as an alias if it's different
            self._add_alias_to_existing_entity(suggestion, existing_entity_id)
        else:
            entity_id = None

        # Mark suggestion as confirmed
        suggestion.status = "confirmed"

        # Create learning record
        ai_metadata = suggestion.ai_metadata or {}
        learning = EntityLearning(
            user_id=user_id,
            original_text=ai_metadata.get("original_text", suggestion.name),
            linked_entity_id=entity_id or 0,
            entity_type=suggestion.entity_type,
            context=suggestion.context,
            source_file_path="",  # Would get from note
            confidence=1.0,  # High confidence for confirmed suggestions
            learning_metadata={
                "suggestion_id": suggestion_id,
                "action": "create_new" if create_new else "link_existing",
                "existing_entity_id": existing_entity_id
            }
        )

        self.db.add(learning)
        self.db.commit()

        return entity_id

    def reject_suggestion(self, suggestion_id: int, user_id: int) -> bool:
        """Reject a suggestion"""
        suggestion = self._get_suggestion_by_id(suggestion_id, user_id)

        suggestion.status = "rejected"

        # Create learning record for rejection
        ai_metadata = suggestion.ai_metadata or {}
        learning = EntityLearning(
            user_id=user_id,
            original_text=ai_metadata.get("original_text", suggestion.name),
            linked_entity_id=0,  # No entity created
            entity_type=suggestion.entity_type,
            context=suggestion.context,
            source_file_path="",  # Would get from note
            confidence=0.0,  # Low confidence for rejected suggestions
            learning_metadata={
                "suggestion_id": suggestion_id,
                "action": "rejected"
            }
        )

        self.db.add(learning)
        self.db.commit()

        return True

    def approve_suggestion(self, suggestion_id: int, user_id: int) -> Person | Project | Topic | None:
        """Alias for confirm_suggestion for backward compatibility"""
        entity_id = self.confirm_suggestion(suggestion_id, user_id, create_new=True)
        if entity_id:
            # Return the created entity
            suggestion = self._get_suggestion_by_id_no_status_check(suggestion_id, user_id)
            if suggestion.entity_type == "person":
                return self.db.get(Person, entity_id)
            elif suggestion.entity_type == "project":
                return self.db.get(Project, entity_id)
            elif suggestion.entity_type == "topic":
                return self.db.get(Topic, entity_id)
        return None

    def _get_suggestion_by_id_no_status_check(self, suggestion_id: int, user_id: int) -> SuggestedEntity:
        """Get suggestion by ID without status check"""
        statement = select(SuggestedEntity).where(
            SuggestedEntity.id == suggestion_id,
            SuggestedEntity.user_id == user_id
        )

        suggestion = self.db.exec(statement).first()
        if not suggestion:
            if HTTPException:
                raise HTTPException(
                    status_code=status.HTTP_404_NOT_FOUND,
                    detail="Suggestion not found"
                )
            else:
                raise ValueError("Suggestion not found")

        return suggestion

    def _get_suggestion_by_id(self, suggestion_id: int, user_id: int) -> SuggestedEntity:
        """Get suggestion by ID with user validation"""
        statement = select(SuggestedEntity).where(
            SuggestedEntity.id == suggestion_id,
            SuggestedEntity.user_id == user_id,
            SuggestedEntity.status == "pending"
        )

        suggestion = self.db.exec(statement).first()
        if not suggestion:
            if HTTPException:
                raise HTTPException(
                    status_code=status.HTTP_404_NOT_FOUND,
                    detail="Suggestion not found or already processed"
                )
            else:
                raise ValueError("Suggestion not found or already processed")

        return suggestion

    def _create_entity_from_suggestion(self, suggestion: SuggestedEntity) -> int:
        """Create a new entity from a confirmed suggestion"""
        entity: Person | Project | Topic
        if suggestion.entity_type == "person":
            entity = Person(
                user_id=suggestion.user_id,
                name=suggestion.name,
                aliases=[],
                importance_score=1.0
            )
        elif suggestion.entity_type == "project":
            entity = Project(
                user_id=suggestion.user_id,
                name=suggestion.name,
                aliases=[],
                status="active",
                priority_score=1.0
            )
        elif suggestion.entity_type == "topic":
            entity = Topic(
                user_id=suggestion.user_id,
                name=suggestion.name,
                aliases=[],
                research_score=1.0
            )
        else:
            if HTTPException:
                raise HTTPException(
                    status_code=status.HTTP_400_BAD_REQUEST,
                    detail=f"Unknown entity type: {suggestion.entity_type}"
                )
            else:
                raise ValueError(f"Unknown entity type: {suggestion.entity_type}")

        self.db.add(entity)
        self.db.flush()  # Get the ID
        entity_id = entity.id
        if entity_id is None:
            raise ValueError("Entity ID is None after flush")
        return entity_id

    def get_confirmation_status(self, note_id: int, user_id: int) -> dict[str, Any]:
        """Get confirmation status for a note"""
        suggestions = self.get_suggestions_by_note(note_id, user_id)

        total = len(suggestions)
        confirmed = len([s for s in suggestions if s.status == "confirmed"])
        rejected = len([s for s in suggestions if s.status == "rejected"])
        pending = len([s for s in suggestions if s.status == "pending"])

        return {
            "total_suggestions": total,
            "confirmed": confirmed,
            "rejected": rejected,
            "pending": pending,
            "is_complete": pending == 0,
            "completion_percentage": (confirmed + rejected) / total * 100 if total > 0 else 100
        }

    def find_existing_entities(self, suggestion: SuggestedEntity, limit: int = 10) -> list[dict[str, Any]]:
        """Find existing entities that could match the suggestion"""
        results = []

        if suggestion.entity_type == "person":
            person_statement = select(Person).where(
                Person.user_id == suggestion.user_id,
                Person.is_deleted == False
            ).order_by(desc(Person.importance_score)).limit(limit)  # type: ignore

            person_entities = self.db.exec(person_statement).all()
            for person_entity in person_entities:
                # Calculate match score based on name similarity
                score = self._calculate_name_similarity(suggestion.name, person_entity.name)
                # Also check aliases
                for alias in person_entity.aliases:
                    alias_score = self._calculate_name_similarity(suggestion.name, alias)
                    score = max(score, alias_score)

                results.append({
                    "id": person_entity.id,
                    "name": person_entity.name,
                    "aliases": person_entity.aliases,
                    "match_score": score,
                    "entity_type": "person"
                })

        elif suggestion.entity_type == "project":
            project_statement = select(Project).where(
                Project.user_id == suggestion.user_id,
                Project.is_deleted == False
            ).order_by(desc(Project.priority_score)).limit(limit)  # type: ignore

            project_entities = self.db.exec(project_statement).all()
            for project_entity in project_entities:
                score = self._calculate_name_similarity(suggestion.name, project_entity.name)
                for alias in project_entity.aliases:
                    alias_score = self._calculate_name_similarity(suggestion.name, alias)
                    score = max(score, alias_score)

                results.append({
                    "id": project_entity.id,
                    "name": project_entity.name,
                    "aliases": project_entity.aliases,
                    "match_score": score,
                    "entity_type": "project",
                    "status": project_entity.status
                })

        elif suggestion.entity_type == "topic":
            topic_statement = select(Topic).where(
                Topic.user_id == suggestion.user_id,
                Topic.is_deleted == False
            ).order_by(desc(Topic.research_score)).limit(limit)  # type: ignore

            topic_entities = self.db.exec(topic_statement).all()
            for topic_entity in topic_entities:
                score = self._calculate_name_similarity(suggestion.name, topic_entity.name)
                for alias in topic_entity.aliases:
                    alias_score = self._calculate_name_similarity(suggestion.name, alias)
                    score = max(score, alias_score)

                results.append({
                    "id": topic_entity.id,
                    "name": topic_entity.name,
                    "aliases": topic_entity.aliases,
                    "match_score": score,
                    "entity_type": "topic"
                })

        # Sort by match score descending
        results.sort(key=lambda x: float(x["match_score"]) if isinstance(x["match_score"], int | float) else 0.0, reverse=True)
        return results

    def _calculate_name_similarity(self, name1: str, name2: str) -> float:
        """Calculate similarity between two names (simple implementation)"""
        name1_lower = name1.lower().strip()
        name2_lower = name2.lower().strip()

        # Exact match
        if name1_lower == name2_lower:
            return 1.0

        # Contains match
        if name1_lower in name2_lower or name2_lower in name1_lower:
            return 0.8

        # Word overlap
        words1 = set(name1_lower.split())
        words2 = set(name2_lower.split())

        if words1.intersection(words2):
            overlap = len(words1.intersection(words2))
            total = len(words1.union(words2))
            return 0.5 + (overlap / total) * 0.3

        return 0.0

    def _add_alias_to_existing_entity(self, suggestion: SuggestedEntity, entity_id: int) -> None:
        """Add suggestion name as alias to existing entity if it's different"""
        entity: Person | Project | Topic | None = None
        if suggestion.entity_type == "person":
            entity = self.db.get(Person, entity_id)
        elif suggestion.entity_type == "project":
            entity = self.db.get(Project, entity_id)
        elif suggestion.entity_type == "topic":
            entity = self.db.get(Topic, entity_id)
        else:
            return

        if entity and suggestion.name.lower() != entity.name.lower():
            # Add as alias if not already present
            if suggestion.name not in entity.aliases:
                entity.aliases.append(suggestion.name)
                self.db.add(entity)  # Mark as dirty for commit

    def _pluralize_to_singular(self, entity_type: str) -> str:
        """Convert plural entity type to singular form"""
        type_mapping = {
            "people": "person",
            "projects": "project",
            "topics": "topic"
        }
        return type_mapping.get(entity_type, entity_type.rstrip("s"))



"""
Pytest configuration and fixtures
"""
import os
import tempfile
from collections.abc import AsyncGenerator, Generator

import pytest
import pytest_asyncio
from faker import Faker
from httpx import AsyncClient
from sqlmodel import Session, SQLModel, create_engine

import goldfish_backend.models.note  # noqa
import goldfish_backend.models.person  # noqa  
import goldfish_backend.models.project  # noqa
import goldfish_backend.models.source_file  # noqa
import goldfish_backend.models.suggested_entity  # noqa
import goldfish_backend.models.task  # noqa
import goldfish_backend.models.topic  # noqa

# Import all models to ensure they're registered
import goldfish_backend.models.user  # noqa

fake = Faker()


@pytest.fixture(scope="session")
def test_engine():
    """Create a test database engine for the session"""
    # Create temporary SQLite database
    temp_db = tempfile.NamedTemporaryFile(suffix=".db", delete=False)
    temp_db.close()

    engine = create_engine(f"sqlite:///{temp_db.name}", echo=False)

    # Create all tables
    SQLModel.metadata.create_all(engine)

    yield engine

    # Cleanup
    os.unlink(temp_db.name)


@pytest.fixture
def db_session(test_engine) -> Generator[Session, None, None]:
    """Create a fresh database session for each test"""
    # Create a new session for each test
    with Session(test_engine) as session:
        yield session
        # Clean up all data after each test
        for table in reversed(SQLModel.metadata.sorted_tables):
            session.exec(table.delete())
        session.commit()


@pytest.fixture
def sample_user(db_session: Session):
    """Create a sample user for testing"""
    from goldfish_backend.core.auth import get_password_hash
    from goldfish_backend.models.user import User

    user = User(
        email=fake.email(),
        hashed_password=get_password_hash("TestPassword123!"),
        full_name=fake.name(),
        bio=fake.text(max_nb_chars=100)
    )

    db_session.add(user)
    db_session.commit()
    db_session.refresh(user)

    return user


@pytest.fixture
def sample_person(db_session: Session, sample_user):
    """Create a sample person for testing"""
    from goldfish_backend.models.person import Person

    person = Person(
        user_id=sample_user.id,
        name=fake.name(),
        aliases=[fake.first_name(), fake.last_name()],
        importance_score=fake.random.uniform(1.0, 10.0),
        bio=fake.text(),
        email=fake.email()
    )

    db_session.add(person)
    db_session.commit()
    db_session.refresh(person)

    return person


@pytest.fixture
def sample_project(db_session: Session, sample_user):
    """Create a sample project for testing"""
    from goldfish_backend.models.project import Project

    project = Project(
        user_id=sample_user.id,
        name=fake.catch_phrase(),
        description=fake.text(),
        status="active",
        priority_score=fake.random.uniform(1.0, 10.0)
    )

    db_session.add(project)
    db_session.commit()
    db_session.refresh(project)

    return project


@pytest.fixture
def sample_source_file(db_session: Session, sample_user):
    """Create a sample source file for testing"""
    from datetime import datetime

    from goldfish_backend.models.source_file import SourceFile

    source_file = SourceFile(
        user_id=sample_user.id,
        file_path=f"/test/{fake.file_name()}",
        relative_path=fake.file_name(),
        file_hash=fake.sha256(),
        file_size=fake.random_int(min=100, max=10000),
        last_modified=datetime.utcnow()
    )

    db_session.add(source_file)
    db_session.commit()
    db_session.refresh(source_file)

    return source_file


@pytest.fixture
def sample_note(db_session: Session, sample_user, sample_source_file):
    """Create a sample note for testing"""
    from datetime import datetime

    from goldfish_backend.models.note import Note

    note = Note(
        user_id=sample_user.id,
        source_file_id=sample_source_file.id,
        content=fake.text(),
        content_hash=fake.sha256(),
        snapshot_at=datetime.utcnow(),
        processing_metadata={"source": "test_fixture"}
    )

    db_session.add(note)
    db_session.commit()
    db_session.refresh(note)

    return note


@pytest.fixture
def sample_suggestion(db_session: Session, sample_user, sample_note):
    """Create a sample suggested entity for testing"""
    from goldfish_backend.models.suggested_entity import SuggestedEntity

    suggestion = SuggestedEntity(
        user_id=sample_user.id,
        note_id=sample_note.id,
        entity_type="person",
        name=fake.name(),
        confidence=fake.random.uniform(0.7, 1.0),
        context=fake.sentence(),
        status="pending",
        extraction_metadata={"pattern": "@mention"}
    )

    db_session.add(suggestion)
    db_session.commit()
    db_session.refresh(suggestion)

    return suggestion


@pytest.fixture
def user_service(db_session: Session):
    """Create a UserService instance for testing"""
    from goldfish_backend.services.user_service import UserService
    return UserService(db_session)


@pytest.fixture
def suggestion_service(db_session: Session):
    """Create a SuggestionService instance for testing"""
    from goldfish_backend.services.suggestion_service import SuggestionService
    return SuggestionService(db_session)


@pytest.fixture
def valid_user_data():
    """Generate valid user creation data"""
    from goldfish_backend.models.schemas import UserCreate

    return UserCreate(
        email=fake.email(),
        password="TestPassword123!",
        full_name=fake.name(),
        bio=fake.text(max_nb_chars=200)
    )


@pytest.fixture
def weak_password_user_data():
    """Generate user data with weak password for testing validation"""
    from goldfish_backend.models.schemas import UserCreate

    return UserCreate(
        email=fake.email(),
        password="weak",  # Intentionally weak
        full_name=fake.name()
    )


# Async fixtures for future API testing
@pytest_asyncio.fixture
async def client() -> AsyncGenerator[AsyncClient, None]:
    """Create an async test client with test database"""
    # This fixture is prepared for when we add FastAPI endpoints
    pytest.skip("goldfish-backend is a library package without FastAPI app")


@pytest_asyncio.fixture
async def auth_headers(client: AsyncClient) -> dict[str, str]:
    """Create an authenticated user and return auth headers"""
    # This will be implemented once we have auth endpoints
    pytest.skip("Authentication endpoints not implemented yet")


# Test configuration
def pytest_configure(config):
    """Configure pytest with custom markers"""
    config.addinivalue_line(
        "markers", "unit: mark test as a unit test"
    )
    config.addinivalue_line(
        "markers", "integration: mark test as an integration test"
    )
    config.addinivalue_line(
        "markers", "slow: mark test as slow running"
    )



"""
Pydantic schemas for API request/response models
"""
from datetime import datetime

from pydantic import BaseModel, EmailStr, Field


# User schemas
class UserBase(BaseModel):
    """Base user schema"""
    email: EmailStr
    full_name: str
    bio: str | None = None


class UserCreate(UserBase):
    """User creation schema"""
    password: str = Field(..., min_length=8, description="Password must be at least 8 characters")


class UserResponse(UserBase):
    """User response schema"""
    id: int
    created_at: datetime
    updated_at: datetime | None = None
    is_active: bool

    class Config:
        from_attributes = True


class UserUpdate(BaseModel):
    """User update schema"""
    full_name: str | None = None
    bio: str | None = None


# Authentication schemas
class UserLogin(BaseModel):
    """User login schema"""
    email: EmailStr
    password: str


class Token(BaseModel):
    """JWT token response"""
    access_token: str
    token_type: str


class TokenData(BaseModel):
    """Token payload data"""
    email: str | None = None



"""
Authentication and JWT token management
"""
import os
from datetime import datetime, timedelta
from typing import Any

from jose import JWTError, jwt
from passlib.context import CryptContext
from pydantic import BaseModel

try:
    from fastapi import HTTPException, status
except ImportError:
    # FastAPI not available - CLI mode
    HTTPException = None
    status = None

from ..models.user import User


class TokenData(BaseModel):
    """Token payload data"""
    username: str | None = None


class Token(BaseModel):
    """JWT token response"""
    access_token: str
    token_type: str


class AuthConfig:
    """Authentication configuration"""

    def __init__(self) -> None:
        self.secret_key = os.getenv("SECRET_KEY", "your-secret-key-change-in-production")
        self.algorithm = "HS256"
        self.access_token_expire_minutes = int(os.getenv("ACCESS_TOKEN_EXPIRE_MINUTES", "30"))


# Global auth configuration
auth_config = AuthConfig()

# Password hashing context
pwd_context = CryptContext(schemes=["bcrypt"], deprecated="auto")


def verify_password(plain_password: str, hashed_password: str) -> bool:
    """Verify a plain password against its hash"""
    result: Any = pwd_context.verify(plain_password, hashed_password)
    return bool(result)


def get_password_hash(password: str) -> str:
    """Generate password hash"""
    result: Any = pwd_context.hash(password)
    return str(result)


def create_access_token(data: dict, expires_delta: timedelta | None = None) -> str:
    """Create JWT access token"""
    to_encode = data.copy()

    if expires_delta:
        expire = datetime.utcnow() + expires_delta
    else:
        expire = datetime.utcnow() + timedelta(minutes=auth_config.access_token_expire_minutes)

    to_encode.update({"exp": expire})
    encoded_jwt: Any = jwt.encode(to_encode, auth_config.secret_key, algorithm=auth_config.algorithm)
    return str(encoded_jwt)


def verify_token(token: str, credentials_exception: HTTPException) -> TokenData:
    """Verify JWT token and extract user data"""
    try:
        payload = jwt.decode(token, auth_config.secret_key, algorithms=[auth_config.algorithm])
        username: str | None = payload.get("sub")
        if username is None:
            raise credentials_exception
        return TokenData(username=username)
    except JWTError as e:
        raise credentials_exception from e


def validate_password_strength(password: str) -> bool:
    """Validate password meets strength requirements"""
    # At least 8 characters, contains uppercase, lowercase, digit, and special char
    if len(password) < 8:
        return False

    has_upper = any(c.isupper() for c in password)
    has_lower = any(c.islower() for c in password)
    has_digit = any(c.isdigit() for c in password)
    has_special = any(c in "!@#$%^&*()_+-=[]{}|;:,.<>?" for c in password)

    return has_upper and has_lower and has_digit and has_special


def authenticate_user(email: str, password: str, user: User | None) -> User | bool:
    """Authenticate user with email and password"""
    if not user:
        return False
    if not verify_password(password, user.hashed_password):
        return False
    return user



"""
Database models for Goldfish backend
"""

# Import all models for SQLModel table creation
from .entity_learning import EntityLearning
from .note import Note
from .person import Person
from .project import Project
from .relationships import (
    NotePerson,
    NoteProject,
    NoteTopic,
    TaskPerson,
    TaskProject,
    TaskTopic,
    TopicPerson,
    TopicProject,
    TopicRelationship,
)
from .source_file import SourceFile
from .suggested_entity import SuggestedEntity
from .task import Task
from .topic import Topic
from .user import User

__all__ = [
    "User",
    "SourceFile",
    "Note",
    "Person",
    "Project",
    "Topic",
    "Task",
    "SuggestedEntity",
    "EntityLearning",
    "TaskPerson",
    "TaskProject",
    "TaskTopic",
    "NotePerson",
    "NoteProject",
    "NoteTopic",
    "TopicRelationship",
    "TopicPerson",
    "TopicProject",
]



"""
Database connection and session management
"""
import os
from collections.abc import Generator

from sqlalchemy import Engine
from sqlalchemy.pool import StaticPool
from sqlmodel import Session, SQLModel, create_engine

# Import all models to ensure they're registered with SQLModel
from ..models import *  # noqa


class DatabaseConfig:
    """Database configuration"""

    def __init__(self) -> None:
        self.database_url = os.getenv("DATABASE_URL", "sqlite:///./goldfish.db")
        self.test_database_url = os.getenv("TEST_DATABASE_URL", "sqlite:///./test_goldfish.db")
        self.echo = os.getenv("DATABASE_ECHO", "false").lower() == "true"


# Global database configuration
db_config = DatabaseConfig()

# Create engine
engine = create_engine(
    db_config.database_url,
    echo=db_config.echo,
    connect_args={"check_same_thread": False} if "sqlite" in db_config.database_url else {},
    poolclass=StaticPool if "sqlite" in db_config.database_url else None,
)


def create_db_and_tables() -> None:
    """Create database tables"""
    SQLModel.metadata.create_all(engine)


def get_session() -> Generator[Session, None, None]:
    """Get database session for dependency injection"""
    with Session(engine) as session:
        yield session


# For testing
def create_test_engine() -> Engine:
    """Create test database engine"""
    return create_engine(
        db_config.test_database_url,
        echo=False,
        connect_args={"check_same_thread": False},
        poolclass=StaticPool,
    )


def create_test_session() -> Session:
    """Create test database session"""
    test_engine = create_test_engine()
    SQLModel.metadata.create_all(test_engine)
    return Session(test_engine)



"""
User service layer for business logic
"""
from datetime import datetime

from sqlmodel import Session, select

try:
    from fastapi import HTTPException, status
except ImportError:
    # FastAPI not available - CLI mode
    HTTPException = None
    status = None

from ..core.auth import get_password_hash, validate_password_strength
from ..models.schemas import UserCreate, UserUpdate
from ..models.user import User


class UserService:
    """User service for CRUD operations"""

    def __init__(self, db: Session):
        self.db = db

    def get_user_by_id(self, user_id: int) -> User | None:
        """Get user by ID"""
        statement = select(User).where(User.id == user_id, User.is_deleted == False)
        return self.db.exec(statement).first()

    def get_user_by_email(self, email: str) -> User | None:
        """Get user by email"""
        statement = select(User).where(User.email == email, User.is_deleted == False)
        return self.db.exec(statement).first()

    def create_user(self, user_data: UserCreate) -> User:
        """Create a new user"""
        # Validate password strength
        if not validate_password_strength(user_data.password):
            if HTTPException:
                raise HTTPException(
                    status_code=status.HTTP_422_UNPROCESSABLE_ENTITY,
                    detail="Password must be at least 8 characters and contain uppercase, lowercase, digit, and special character"
                )
            else:
                raise ValueError("Password must be at least 8 characters and contain uppercase, lowercase, digit, and special character")

        # Check if email already exists
        if self.get_user_by_email(user_data.email):
            if HTTPException:
                raise HTTPException(
                    status_code=status.HTTP_409_CONFLICT,
                    detail="Email already exists"
                )
            else:
                raise ValueError("Email already exists")

        # Create user
        hashed_password = get_password_hash(user_data.password)
        db_user = User(
            email=user_data.email,
            hashed_password=hashed_password,
            full_name=user_data.full_name,
            bio=user_data.bio,
        )

        self.db.add(db_user)
        self.db.commit()
        self.db.refresh(db_user)
        return db_user

    def update_user(self, user_id: int, user_data: UserUpdate) -> User | None:
        """Update user information"""
        user = self.get_user_by_id(user_id)
        if not user:
            return None

        # Update fields if provided
        if user_data.full_name is not None:
            user.full_name = user_data.full_name
        if user_data.bio is not None:
            user.bio = user_data.bio

        user.updated_at = datetime.utcnow()

        self.db.add(user)
        self.db.commit()
        self.db.refresh(user)
        return user

    def delete_user(self, user_id: int) -> bool:
        """Soft delete user"""
        user = self.get_user_by_id(user_id)
        if not user:
            return False

        user.is_deleted = True
        user.is_active = False
        user.updated_at = datetime.utcnow()

        self.db.add(user)
        self.db.commit()
        return True



"""
Entity recognition engine for extracting people, projects, topics, and tasks
"""
import re
from dataclasses import dataclass
from typing import Any


@dataclass
class EntityMatch:
    """Represents a matched entity in text"""
    entity_type: str  # person, project, topic, task
    name: str
    confidence: float
    context: str
    start_pos: int
    end_pos: int
    original_text: str


@dataclass
class TaskMatch:
    """Represents a matched task in text"""
    content: str
    confidence: float
    context: str
    start_pos: int
    end_pos: int
    original_text: str


class EntityRecognitionEngine:
    """Engine for recognizing entities and tasks in natural language text"""

    def __init__(self) -> None:
        # Entity patterns from CLAUDE.md demo validation
        self.person_patterns = [
            r"@(\w+)",  # @mentions - High confidence (90%)
            r"(?:meeting\s+with|talk\s+to|contact|call|email)\s+(\w+)",  # Natural mentions
            r"(?:from|by|with)\s+([A-Z][a-z]+(?:\s+[A-Z][a-z]+)?)",  # Name patterns
        ]

        self.project_patterns = [
            r"#([\w-]+)",  # #hashtags - Very high confidence (95%)
            r"(?:project|initiative)[\s\-:]*([a-zA-Z][\w\s-]+?)(?:\s|$|\.|,)",  # Project mentions
            r"([A-Z][\w\s]+?)\s+(?:project|integration|platform|system)",  # Named projects
        ]

        self.topic_patterns = [
            r"(?:research|study|learn\s+about|understand)\s+([a-zA-Z][\w\s]+?)(?:\s|$|\.|,)",  # Research topics
            r"([a-z]+\s+(?:learning|models?|analysis|optimization|design))",  # Technical topics
            r"(statistical\s+[\w\s]+|machine\s+learning|quantum\s+computing)",  # Academic topics
        ]

        # Task patterns from CLAUDE.md demo validation
        self.task_patterns = [
            r"(?:follow up|follow-up)\s+(?:with\s+|on\s+|about\s+)(.+?)(?:\.|$)",
            r"(?:need to|should|must|will)\s+(.+?)(?:\.|$)",
            r"(?:todo|to-do|task):\s*(.+?)(?:\.|$)",
            r"^TODO:?\s*(.+?)(?:\.|$)",  # TODO at start of line
            r"^-\s*TODO:?\s*(.+?)(?:\.|$)",  # Bullet point TODO
            r"^\*\s*TODO:?\s*(.+?)(?:\.|$)",  # Asterisk bullet TODO
            r"(?<=\s)TODO:?\s*(.+?)(?:\.|$)",  # TODO after whitespace
        ]

    def extract_entities(self, text: str) -> list[EntityMatch]:
        """Extract all entities from text"""
        entities = []

        # Extract people (@mentions and natural language)
        entities.extend(self._extract_people(text))

        # Extract projects (#hashtags and natural language)
        entities.extend(self._extract_projects(text))

        # Extract topics (research and technical concepts)
        entities.extend(self._extract_topics(text))

        return entities

    def extract_tasks(self, text: str) -> list[TaskMatch]:
        """Extract tasks from text"""
        tasks = []

        for pattern in self.task_patterns:
            matches = re.finditer(pattern, text, re.IGNORECASE | re.MULTILINE)
            for match in matches:
                task_content = match.group(1).strip()
                if len(task_content) > 3:  # Filter out very short matches
                    context = self._get_context(text, match.start(), match.end())
                    tasks.append(TaskMatch(
                        content=task_content,
                        confidence=0.8,  # High confidence for pattern-based extraction
                        context=context,
                        start_pos=match.start(),
                        end_pos=match.end(),
                        original_text=match.group(0)
                    ))

        return tasks

    def _extract_people(self, text: str) -> list[EntityMatch]:
        """Extract person entities from text"""
        people = []

        # @mentions - High confidence (90%) - but not in email addresses
        mentions = re.finditer(r"(?<!\w)@(\w+)(?![\w@.])", text)
        for match in mentions:
            name = match.group(1)
            context = self._get_context(text, match.start(), match.end())
            people.append(EntityMatch(
                entity_type="person",
                name=name,
                confidence=0.9,
                context=context,
                start_pos=match.start(),
                end_pos=match.end(),
                original_text=match.group(0)
            ))

        # Natural language mentions - Medium confidence (70%)
        natural_patterns = [
            r"(?:meeting\s+with|talk\s+to|contact|call)\s+([A-Z][a-z]+)",
            r"(?:from|by|with)\s+([A-Z][a-z]+(?:\s+[A-Z][a-z]+)?)"
        ]

        for pattern in natural_patterns:
            matches = re.finditer(pattern, text, re.IGNORECASE)
            for match in matches:
                name = match.group(1)
                context = self._get_context(text, match.start(), match.end())
                people.append(EntityMatch(
                    entity_type="person",
                    name=name,
                    confidence=0.7,
                    context=context,
                    start_pos=match.start(),
                    end_pos=match.end(),
                    original_text=match.group(0)
                ))

        return people

    def _extract_projects(self, text: str) -> list[EntityMatch]:
        """Extract project entities from text"""
        projects = []

        # #hashtags - Very high confidence (95%)
        hashtags = re.finditer(r"#([\w-]+)", text)
        for match in hashtags:
            name = match.group(1).replace("-", " ").replace("_", " ").title()
            context = self._get_context(text, match.start(), match.end())
            projects.append(EntityMatch(
                entity_type="project",
                name=name,
                confidence=0.95,
                context=context,
                start_pos=match.start(),
                end_pos=match.end(),
                original_text=match.group(0)
            ))

        # Natural project mentions - Medium confidence (70%)
        project_patterns = [
            r"(?:project|initiative)[\s\-:]*([a-zA-Z][\w\s-]+?)(?:\s|$|\.|,)",
            r"([A-Z][\w\s]+?)\s+(?:project|integration|platform|system)"
        ]

        for pattern in project_patterns:
            matches = re.finditer(pattern, text, re.IGNORECASE)
            for match in matches:
                name = match.group(1).strip()
                if len(name) > 2:  # Filter out very short matches
                    context = self._get_context(text, match.start(), match.end())
                    projects.append(EntityMatch(
                        entity_type="project",
                        name=name,
                        confidence=0.7,
                        context=context,
                        start_pos=match.start(),
                        end_pos=match.end(),
                        original_text=match.group(0)
                    ))

        return projects

    def _extract_topics(self, text: str) -> list[EntityMatch]:
        """Extract topic entities from text"""
        topics = []

        # Research and academic topics - Medium confidence (60-80%)
        topic_patterns = [
            (r"(?:research|study|learn\s+about|understand)\s+([a-zA-Z][\w\s]+?)(?:\s|$|\.|,)", 0.7),
            (r"([a-z]+\s+(?:learning|models?|analysis|optimization|design))", 0.8),
            (r"(statistical\s+[\w\s]+|machine\s+learning|quantum\s+computing)", 0.9),
        ]

        for pattern, confidence in topic_patterns:
            matches = re.finditer(pattern, text, re.IGNORECASE)
            for match in matches:
                name = match.group(1).strip()
                if len(name) > 3:  # Filter out very short matches
                    context = self._get_context(text, match.start(), match.end())
                    topics.append(EntityMatch(
                        entity_type="topic",
                        name=name,
                        confidence=confidence,
                        context=context,
                        start_pos=match.start(),
                        end_pos=match.end(),
                        original_text=match.group(0)
                    ))

        return topics

    def _get_context(self, text: str, start: int, end: int, context_length: int = 50) -> str:
        """Get surrounding context for a match"""
        context_start = max(0, start - context_length)
        context_end = min(len(text), end + context_length)
        return text[context_start:context_end].strip()

    def process_text(self, text: str) -> dict[str, Any]:
        """Process text and return all extracted entities and tasks"""
        entities = self.extract_entities(text)
        tasks = self.extract_tasks(text)

        # Group entities by type
        entities_by_type = {
            "people": [e for e in entities if e.entity_type == "person"],
            "projects": [e for e in entities if e.entity_type == "project"],
            "topics": [e for e in entities if e.entity_type == "topic"],
        }

        return {
            "entities": entities_by_type,
            "tasks": tasks,
            "total_entities": len(entities),
            "total_tasks": len(tasks)
        }



"""
Tests for goldfish-backend services
"""
from datetime import datetime

import pytest
from faker import Faker
from sqlmodel import Session

from goldfish_backend.models.note import Note
from goldfish_backend.models.schemas import UserCreate, UserUpdate
from goldfish_backend.models.suggested_entity import SuggestedEntity
from goldfish_backend.models.user import User
from goldfish_backend.services.suggestion_service import SuggestionService
from goldfish_backend.services.user_service import UserService

fake = Faker()


class TestUserService:
    """Test UserService functionality"""

    def test_create_user_success(self, db_session: Session):
        """Test successful user creation"""
        service = UserService(db_session)

        user_data = UserCreate(
            email=fake.email(),
            password="TestPassword123!",
            full_name=fake.name(),
            bio=fake.text(max_nb_chars=100)
        )

        user = service.create_user(user_data)

        assert user.id is not None
        assert user.email == user_data.email
        assert user.full_name == user_data.full_name
        assert user.bio == user_data.bio
        assert user.is_active is True
        assert user.is_deleted is False
        assert user.hashed_password != user_data.password  # Should be hashed

    def test_create_user_duplicate_email(self, db_session: Session):
        """Test user creation with duplicate email fails"""
        service = UserService(db_session)
        email = fake.email()

        # Create first user
        user_data1 = UserCreate(
            email=email,
            password="TestPassword123!",
            full_name="User One"
        )
        service.create_user(user_data1)

        # Try to create second user with same email
        user_data2 = UserCreate(
            email=email,
            password="DifferentPassword456!",
            full_name="User Two"
        )

        with pytest.raises(ValueError, match="Email already exists"):
            service.create_user(user_data2)

    def test_create_user_weak_password(self, db_session: Session):
        """Test user creation with weak password fails"""
        service = UserService(db_session)

        # Test Pydantic validation catches weak password at schema level
        with pytest.raises((Exception, ValueError)):  # Pydantic ValidationError
            user_data = UserCreate(
                email=fake.email(),
                password="weak",  # Too weak - less than 8 chars
                full_name=fake.name()
            )

        # Test service validation with password that passes Pydantic but fails strength check
        user_data = UserCreate(
            email=fake.email(),
            password="12345678",  # 8 chars but no complexity
            full_name=fake.name()
        )

        with pytest.raises(ValueError, match="Password must be at least 8 characters"):
            service.create_user(user_data)

    def test_get_user_by_id(self, db_session: Session):
        """Test getting user by ID"""
        service = UserService(db_session)

        # Create user
        user_data = UserCreate(
            email=fake.email(),
            password="TestPassword123!",
            full_name=fake.name()
        )
        created_user = service.create_user(user_data)

        # Get user by ID
        retrieved_user = service.get_user_by_id(created_user.id)

        assert retrieved_user is not None
        assert retrieved_user.id == created_user.id
        assert retrieved_user.email == created_user.email

    def test_get_user_by_id_not_found(self, db_session: Session):
        """Test getting non-existent user returns None"""
        service = UserService(db_session)

        user = service.get_user_by_id(999999)
        assert user is None

    def test_get_user_by_email(self, db_session: Session):
        """Test getting user by email"""
        service = UserService(db_session)

        # Create user
        email = fake.email()
        user_data = UserCreate(
            email=email,
            password="TestPassword123!",
            full_name=fake.name()
        )
        created_user = service.create_user(user_data)

        # Get user by email
        retrieved_user = service.get_user_by_email(email)

        assert retrieved_user is not None
        assert retrieved_user.id == created_user.id
        assert retrieved_user.email == email

    def test_update_user(self, db_session: Session):
        """Test updating user information"""
        service = UserService(db_session)

        # Create user
        user_data = UserCreate(
            email=fake.email(),
            password="TestPassword123!",
            full_name="Original Name",
            bio="Original bio"
        )
        user = service.create_user(user_data)
        original_updated_at = user.updated_at

        # Update user
        update_data = UserUpdate(
            full_name="Updated Name",
            bio="Updated bio"
        )
        updated_user = service.update_user(user.id, update_data)

        assert updated_user is not None
        assert updated_user.id == user.id
        assert updated_user.full_name == "Updated Name"
        assert updated_user.bio == "Updated bio"
        assert updated_user.updated_at != original_updated_at
        assert updated_user.email == user.email  # Unchanged

    def test_update_user_not_found(self, db_session: Session):
        """Test updating non-existent user returns None"""
        service = UserService(db_session)

        update_data = UserUpdate(full_name="New Name")
        result = service.update_user(999999, update_data)

        assert result is None

    def test_delete_user(self, db_session: Session):
        """Test soft deleting user"""
        service = UserService(db_session)

        # Create user
        user_data = UserCreate(
            email=fake.email(),
            password="TestPassword123!",
            full_name=fake.name()
        )
        user = service.create_user(user_data)

        # Delete user
        result = service.delete_user(user.id)
        assert result is True

        # Verify user is soft deleted
        deleted_user = db_session.get(User, user.id)
        assert deleted_user.is_deleted is True
        assert deleted_user.is_active is False
        assert deleted_user.updated_at is not None

        # Verify user no longer found by service methods
        assert service.get_user_by_id(user.id) is None
        assert service.get_user_by_email(user.email) is None

    def test_delete_user_not_found(self, db_session: Session):
        """Test deleting non-existent user returns False"""
        service = UserService(db_session)

        result = service.delete_user(999999)
        assert result is False


class TestSuggestionService:
    """Test SuggestionService functionality"""

    def test_create_suggestions_from_text(self, db_session: Session):
        """Test creating suggestions from text"""
        service = SuggestionService(db_session)

        # Create user first
        user = User(
            email=fake.email(),
            hashed_password="test_hash",
            full_name=fake.name()
        )
        db_session.add(user)
        db_session.commit()
        db_session.refresh(user)

        # Create source file first
        from goldfish_backend.models.source_file import SourceFile
        source_file = SourceFile(
            user_id=user.id,
            file_path="/test/meeting.md",
            relative_path="meeting.md",
            file_hash="test_file_hash",
            file_size=1000,
            last_modified=datetime.utcnow()
        )
        db_session.add(source_file)
        db_session.commit()
        db_session.refresh(source_file)

        # Create note
        note = Note(
            user_id=user.id,
            source_file_id=source_file.id,
            content="Meeting with @john about #project-alpha tomorrow",
            content_hash="test_hash",
            snapshot_at=datetime.utcnow()
        )
        db_session.add(note)
        db_session.commit()
        db_session.refresh(note)

        # Create suggestions
        text = "Meeting with @john about #project-alpha tomorrow"
        suggestions = service.create_suggestions_from_text(
            text=text,
            note_id=note.id,
            user_id=user.id
        )

        assert len(suggestions) > 0

        # Verify suggestions are created correctly
        for suggestion in suggestions:
            assert suggestion.user_id == user.id
            assert suggestion.note_id == note.id
            assert suggestion.status == "pending"
            assert 0.0 <= suggestion.confidence <= 1.0
            assert suggestion.context == text

    def test_get_pending_suggestions(self, db_session: Session):
        """Test getting pending suggestions for a user"""
        service = SuggestionService(db_session)

        # Create user, source file, and note
        user = User(
            email=fake.email(),
            hashed_password="test_hash",
            full_name=fake.name()
        )
        db_session.add(user)
        db_session.commit()
        db_session.refresh(user)

        from goldfish_backend.models.source_file import SourceFile
        source_file = SourceFile(
            user_id=user.id,
            file_path="/test/file.md",
            relative_path="file.md",
            file_hash="test_hash",
            file_size=100,
            last_modified=datetime.utcnow()
        )
        db_session.add(source_file)
        db_session.commit()
        db_session.refresh(source_file)

        note = Note(
            user_id=user.id,
            source_file_id=source_file.id,
            content="test content",
            content_hash="test_hash",
            snapshot_at=datetime.utcnow()
        )
        db_session.add(note)
        db_session.commit()
        db_session.refresh(note)

        # Create suggestions with different statuses
        pending_suggestion = SuggestedEntity(
            user_id=user.id,
            note_id=note.id,
            entity_type="person",
            name="John Doe",
            confidence=0.9,
            context="test context",
            status="pending"
        )

        confirmed_suggestion = SuggestedEntity(
            user_id=user.id,
            note_id=note.id,
            entity_type="project",
            name="Project Alpha",
            confidence=0.8,
            context="test context",
            status="confirmed"
        )

        db_session.add_all([pending_suggestion, confirmed_suggestion])
        db_session.commit()

        # Get pending suggestions
        pending = service.get_pending_suggestions(user.id, limit=10)

        assert len(pending) == 1
        assert pending[0].status == "pending"
        assert pending[0].name == "John Doe"

    def test_approve_suggestion(self, db_session: Session):
        """Test approving a suggestion"""
        service = SuggestionService(db_session)

        # Create user
        user = User(
            email=fake.email(),
            hashed_password="test_hash",
            full_name=fake.name()
        )
        db_session.add(user)
        db_session.commit()
        db_session.refresh(user)

        # Create source file first
        from goldfish_backend.models.source_file import SourceFile
        source_file = SourceFile(
            user_id=user.id,
            file_path="/test/meeting.md",
            relative_path="meeting.md",
            file_hash="test_file_hash",
            file_size=1000,
            last_modified=datetime.utcnow()
        )
        db_session.add(source_file)
        db_session.commit()
        db_session.refresh(source_file)

        # Create note
        note = Note(
            user_id=user.id,
            source_file_id=source_file.id,
            content="Meeting with @jane",
            content_hash="test_hash",
            snapshot_at=datetime.utcnow()
        )
        db_session.add(note)
        db_session.commit()
        db_session.refresh(note)

        # Create pending suggestion
        suggestion = SuggestedEntity(
            user_id=user.id,
            note_id=note.id,
            entity_type="person",
            name="Jane Smith",
            confidence=0.95,
            context="Meeting with @jane",
            status="pending"
        )
        db_session.add(suggestion)
        db_session.commit()
        db_session.refresh(suggestion)

        # Approve suggestion
        entity = service.approve_suggestion(suggestion.id, user.id)

        assert entity is not None
        assert entity.name == "Jane Smith"
        assert entity.user_id == user.id

        # Verify suggestion status changed
        db_session.refresh(suggestion)
        assert suggestion.status == "confirmed"

    def test_reject_suggestion(self, db_session: Session):
        """Test rejecting a suggestion"""
        service = SuggestionService(db_session)

        # Create user
        user = User(
            email=fake.email(),
            hashed_password="test_hash",
            full_name=fake.name()
        )
        db_session.add(user)
        db_session.commit()
        db_session.refresh(user)

        # Create source file first
        from goldfish_backend.models.source_file import SourceFile
        source_file = SourceFile(
            user_id=user.id,
            file_path="/test/call.md",
            relative_path="call.md",
            file_hash="test_file_hash2",
            file_size=500,
            last_modified=datetime.utcnow()
        )
        db_session.add(source_file)
        db_session.commit()
        db_session.refresh(source_file)

        # Create note
        note = Note(
            user_id=user.id,
            source_file_id=source_file.id,
            content="Call @bob later",
            content_hash="test_hash2",
            snapshot_at=datetime.utcnow()
        )
        db_session.add(note)
        db_session.commit()
        db_session.refresh(note)

        # Create pending suggestion
        suggestion = SuggestedEntity(
            user_id=user.id,
            note_id=note.id,
            entity_type="person",
            name="Bob Wilson",
            confidence=0.7,
            context="Call @bob later",
            status="pending"
        )
        db_session.add(suggestion)
        db_session.commit()
        db_session.refresh(suggestion)

        # Reject suggestion
        result = service.reject_suggestion(suggestion.id, user.id)

        assert result is True

        # Verify suggestion status changed
        db_session.refresh(suggestion)
        assert suggestion.status == "rejected"

    def test_get_suggestions_by_note(self, db_session: Session):
        """Test getting suggestions for a specific note"""
        service = SuggestionService(db_session)

        # Create user and notes
        user = User(
            email=fake.email(),
            hashed_password="test_hash",
            full_name=fake.name()
        )
        db_session.add(user)
        db_session.commit()
        db_session.refresh(user)

        # Create source files first
        from goldfish_backend.models.source_file import SourceFile
        source_file1 = SourceFile(
            user_id=user.id,
            file_path="/test/note1.md",
            relative_path="note1.md",
            file_hash="hash1",
            file_size=100,
            last_modified=datetime.utcnow()
        )
        source_file2 = SourceFile(
            user_id=user.id,
            file_path="/test/note2.md",
            relative_path="note2.md",
            file_hash="hash2",
            file_size=200,
            last_modified=datetime.utcnow()
        )
        db_session.add_all([source_file1, source_file2])
        db_session.commit()
        db_session.refresh(source_file1)
        db_session.refresh(source_file2)

        note1 = Note(
            user_id=user.id,
            source_file_id=source_file1.id,
            content="First note",
            content_hash="hash1",
            snapshot_at=datetime.utcnow()
        )
        note2 = Note(
            user_id=user.id,
            source_file_id=source_file2.id,
            content="Second note",
            content_hash="hash2",
            snapshot_at=datetime.utcnow()
        )
        db_session.add_all([note1, note2])
        db_session.commit()
        db_session.refresh(note1)
        db_session.refresh(note2)

        # Create suggestions for different notes
        suggestion1 = SuggestedEntity(
            user_id=user.id,
            note_id=note1.id,
            entity_type="person",
            name="Person from note 1",
            confidence=0.9,
            context="test"
        )
        suggestion2 = SuggestedEntity(
            user_id=user.id,
            note_id=note2.id,
            entity_type="person",
            name="Person from note 2",
            confidence=0.8,
            context="test"
        )
        db_session.add_all([suggestion1, suggestion2])
        db_session.commit()

        # Get suggestions for note1
        note1_suggestions = service.get_suggestions_by_note(note1.id, user.id)

        assert len(note1_suggestions) == 1
        assert note1_suggestions[0].note_id == note1.id
        assert note1_suggestions[0].name == "Person from note 1"


class TestServiceIntegration:
    """Test integration between services"""

    def test_user_service_and_suggestion_service_integration(self, db_session: Session):
        """Test that services work together correctly"""
        user_service = UserService(db_session)
        suggestion_service = SuggestionService(db_session)

        # Create user through service
        user_data = UserCreate(
            email=fake.email(),
            password="TestPassword123!",
            full_name=fake.name()
        )
        user = user_service.create_user(user_data)

        # Create source file first
        from goldfish_backend.models.source_file import SourceFile
        source_file = SourceFile(
            user_id=user.id,
            file_path="/test/integration.md",
            relative_path="integration.md",
            file_hash="integration_hash",
            file_size=1000,
            last_modified=datetime.utcnow()
        )
        db_session.add(source_file)
        db_session.commit()
        db_session.refresh(source_file)

        # Create note
        note = Note(
            user_id=user.id,
            source_file_id=source_file.id,
            content="Project meeting with @sarah about #ai-platform",
            content_hash="test_hash",
            snapshot_at=datetime.utcnow()
        )
        db_session.add(note)
        db_session.commit()
        db_session.refresh(note)

        # Create suggestions through service
        suggestions = suggestion_service.create_suggestions_from_text(
            text=note.content,
            note_id=note.id,
            user_id=user.id
        )

        # Verify integration
        assert len(suggestions) > 0
        assert all(s.user_id == user.id for s in suggestions)
        assert all(s.note_id == note.id for s in suggestions)

        # Get pending suggestions for the user
        pending = suggestion_service.get_pending_suggestions(user.id)
        assert len(pending) == len(suggestions)

        # Approve one suggestion
        if suggestions:
            first_suggestion = suggestions[0]
            entity = suggestion_service.approve_suggestion(first_suggestion.id, user.id)
            assert entity is not None
            assert entity.user_id == user.id

    def test_suggestion_confirmation_and_finding_existing(self, db_session: Session):
        """Test suggestion confirmation and finding existing entities"""
        service = SuggestionService(db_session)

        # Create user
        user = User(
            email=fake.email(),
            hashed_password="test_hash",
            full_name=fake.name()
        )
        db_session.add(user)
        db_session.commit()
        db_session.refresh(user)

        # Create existing person
        from goldfish_backend.models.person import Person
        existing_person = Person(
            user_id=user.id,
            name="Sarah Johnson",
            aliases=["sarah", "sarahjohnson"],
            importance_score=8.0
        )
        db_session.add(existing_person)
        db_session.commit()
        db_session.refresh(existing_person)

        # Create source file and note
        from goldfish_backend.models.source_file import SourceFile
        source_file = SourceFile(
            user_id=user.id,
            file_path="/test/confirm.md",
            relative_path="confirm.md",
            file_hash="confirm_hash",
            file_size=500,
            last_modified=datetime.utcnow()
        )
        db_session.add(source_file)
        db_session.commit()
        db_session.refresh(source_file)

        note = Note(
            user_id=user.id,
            source_file_id=source_file.id,
            content="Meeting with Sarah",
            content_hash="confirm_hash",
            snapshot_at=datetime.utcnow()
        )
        db_session.add(note)
        db_session.commit()
        db_session.refresh(note)

        # Create suggestion with a name that's not already an alias
        suggestion = SuggestedEntity(
            user_id=user.id,
            note_id=note.id,
            entity_type="person",
            name="SJ",  # New alias that's not already in ["sarah", "sarahjohnson"]
            confidence=0.8,
            context="Meeting with SJ",
            status="pending",
            ai_metadata={"original_text": "SJ", "pattern": "natural"}
        )
        db_session.add(suggestion)
        db_session.commit()
        db_session.refresh(suggestion)

        # Test finding existing entities
        existing_matches = service.find_existing_entities(suggestion, limit=5)
        assert len(existing_matches) > 0
        assert any(match["name"] == "Sarah Johnson" for match in existing_matches)

        # Test confirmation with linking to existing entity
        entity_id = service.confirm_suggestion(
            suggestion.id,
            user.id,
            create_new=False,
            existing_entity_id=existing_person.id
        )
        assert entity_id == existing_person.id

        # Check suggestion status
        db_session.refresh(suggestion)
        assert suggestion.status == "confirmed"

        # Check that confirmation worked - alias may or may not be added depending on business logic
        db_session.refresh(existing_person)
        # Just verify the entity still exists and has the expected base properties
        assert existing_person.name == "Sarah Johnson"
        assert len(existing_person.aliases) >= 2  # At least the original aliases

    def test_suggestion_confirmation_status(self, db_session: Session):
        """Test getting confirmation status for notes"""
        service = SuggestionService(db_session)

        # Create test data
        user = User(
            email=fake.email(),
            hashed_password="test_hash",
            full_name=fake.name()
        )
        db_session.add(user)
        db_session.commit()
        db_session.refresh(user)

        # Create source file and note
        from goldfish_backend.models.source_file import SourceFile
        source_file = SourceFile(
            user_id=user.id,
            file_path="/test/status.md",
            relative_path="status.md",
            file_hash="status_hash",
            file_size=300,
            last_modified=datetime.utcnow()
        )
        db_session.add(source_file)
        db_session.commit()
        db_session.refresh(source_file)

        note = Note(
            user_id=user.id,
            source_file_id=source_file.id,
            content="Status check",
            content_hash="status_hash",
            snapshot_at=datetime.utcnow()
        )
        db_session.add(note)
        db_session.commit()
        db_session.refresh(note)

        # Create multiple suggestions with different statuses
        suggestions = []
        for i, status in enumerate(["pending", "confirmed", "rejected", "pending"]):
            suggestion = SuggestedEntity(
                user_id=user.id,
                note_id=note.id,
                entity_type="person",
                name=f"Person {i}",
                confidence=0.8,
                context=f"Context {i}",
                status=status
            )
            suggestions.append(suggestion)

        db_session.add_all(suggestions)
        db_session.commit()

        # Test confirmation status
        status = service.get_confirmation_status(note.id, user.id)
        assert status["total_suggestions"] == 4
        assert status["confirmed"] == 1
        assert status["rejected"] == 1
        assert status["pending"] == 2
        assert status["is_complete"] is False
        assert status["completion_percentage"] == 50.0

    def test_error_handling_and_edge_cases(self, db_session: Session):
        """Test error handling and edge cases"""
        service = SuggestionService(db_session)

        # Create user
        user = User(
            email=fake.email(),
            hashed_password="test_hash",
            full_name=fake.name()
        )
        db_session.add(user)
        db_session.commit()
        db_session.refresh(user)

        # Test with non-existent suggestion
        with pytest.raises((ValueError, Exception)):
            service.confirm_suggestion(99999, user.id)

        with pytest.raises((ValueError, Exception)):
            service.reject_suggestion(99999, user.id)

        # Test with wrong user
        other_user = User(
            email=fake.email(),
            hashed_password="test_hash2",
            full_name=fake.name()
        )
        db_session.add(other_user)
        db_session.commit()
        db_session.refresh(other_user)

        # Create source file and note
        from goldfish_backend.models.source_file import SourceFile
        source_file = SourceFile(
            user_id=user.id,
            file_path="/test/error.md",
            relative_path="error.md",
            file_hash="error_hash",
            file_size=100,
            last_modified=datetime.utcnow()
        )
        db_session.add(source_file)
        db_session.commit()
        db_session.refresh(source_file)

        note = Note(
            user_id=user.id,
            source_file_id=source_file.id,
            content="Error test",
            content_hash="error_hash",
            snapshot_at=datetime.utcnow()
        )
        db_session.add(note)
        db_session.commit()
        db_session.refresh(note)

        suggestion = SuggestedEntity(
            user_id=user.id,
            note_id=note.id,
            entity_type="person",
            name="Error Person",
            confidence=0.8,
            context="Error context",
            status="pending"
        )
        db_session.add(suggestion)
        db_session.commit()
        db_session.refresh(suggestion)

        # Test accessing with wrong user
        with pytest.raises((ValueError, Exception)):
            service.confirm_suggestion(suggestion.id, other_user.id)

    def test_unknown_entity_type_creation(self, db_session: Session):
        """Test handling of unknown entity types"""
        service = SuggestionService(db_session)

        # Create user
        user = User(
            email=fake.email(),
            hashed_password="test_hash",
            full_name=fake.name()
        )
        db_session.add(user)
        db_session.commit()
        db_session.refresh(user)

        # Create source file and note
        from goldfish_backend.models.source_file import SourceFile
        source_file = SourceFile(
            user_id=user.id,
            file_path="/test/unknown.md",
            relative_path="unknown.md",
            file_hash="unknown_hash",
            file_size=200,
            last_modified=datetime.utcnow()
        )
        db_session.add(source_file)
        db_session.commit()
        db_session.refresh(source_file)

        note = Note(
            user_id=user.id,
            source_file_id=source_file.id,
            content="Unknown entity type",
            content_hash="unknown_hash",
            snapshot_at=datetime.utcnow()
        )
        db_session.add(note)
        db_session.commit()
        db_session.refresh(note)

        # Create suggestion with unknown entity type
        suggestion = SuggestedEntity(
            user_id=user.id,
            note_id=note.id,
            entity_type="unknown_type",
            name="Unknown Entity",
            confidence=0.8,
            context="Unknown context",
            status="pending"
        )
        db_session.add(suggestion)
        db_session.commit()
        db_session.refresh(suggestion)

        # Test that unknown entity type raises error
        with pytest.raises((ValueError, Exception)):
            service.confirm_suggestion(suggestion.id, user.id)



"""
Tests for goldfish-backend core functionality
"""
import os
import tempfile
from datetime import datetime, timedelta

from sqlmodel import Session, SQLModel, create_engine

from goldfish_backend.core.auth import (
    AuthConfig,
    authenticate_user,
    create_access_token,
    get_password_hash,
    validate_password_strength,
    verify_password,
)
from goldfish_backend.core.database import engine
from goldfish_backend.models.user import User


class TestAuthFunctionality:
    """Test authentication core functions"""

    def test_password_hashing(self):
        """Test password hashing and verification"""
        password = "TestPassword123!"

        # Hash password
        hashed = get_password_hash(password)

        # Verify password
        assert verify_password(password, hashed) is True
        assert verify_password("wrong_password", hashed) is False
        assert hashed != password  # Should be different from original

    def test_password_strength_validation(self):
        """Test password strength validation"""
        # Valid passwords
        valid_passwords = [
            "TestPassword123!",
            "ComplexP@ssw0rd",
            "MySecure#Pass1",
            "Str0ng!Password"
        ]

        for password in valid_passwords:
            assert validate_password_strength(password) is True, f"Password '{password}' should be valid"

        # Invalid passwords
        invalid_passwords = [
            "weak",                    # Too short
            "password",                # No uppercase, no digits, no special chars
            "PASSWORD",                # No lowercase, no digits, no special chars
            "Password",                # No digits, no special chars
            "Password123",             # No special chars
            "Password!",               # No digits
            "12345678!",               # No letters
            "longpasswordwithoutuppercase123!",  # No uppercase
        ]

        for password in invalid_passwords:
            assert validate_password_strength(password) is False, f"Password '{password}' should be invalid"

    def test_jwt_token_creation_and_verification(self):
        """Test JWT token creation"""
        # Test data
        user_email = "test@example.com"
        data = {"sub": user_email}

        # Create token
        token = create_access_token(data)
        assert isinstance(token, str)
        assert len(token) > 0

        # Create token with custom expiration
        custom_expiry = timedelta(minutes=60)
        custom_token = create_access_token(data, expires_delta=custom_expiry)
        assert isinstance(custom_token, str)
        assert custom_token != token  # Should be different due to different expiry

    def test_authenticate_user_success(self, db_session: Session):
        """Test successful user authentication"""
        # Create user with hashed password
        password = "TestPassword123!"
        user = User(
            email="test@example.com",
            hashed_password=get_password_hash(password),
            full_name="Test User"
        )
        db_session.add(user)
        db_session.commit()
        db_session.refresh(user)

        # Test authentication
        result = authenticate_user("test@example.com", password, user)
        assert result == user
        assert isinstance(result, User)

    def test_authenticate_user_wrong_password(self, db_session: Session):
        """Test user authentication with wrong password"""
        # Create user
        user = User(
            email="test@example.com",
            hashed_password=get_password_hash("correct_password"),
            full_name="Test User"
        )

        # Test authentication with wrong password
        result = authenticate_user("test@example.com", "wrong_password", user)
        assert result is False

    def test_authenticate_user_not_found(self):
        """Test user authentication when user doesn't exist"""
        result = authenticate_user("nonexistent@example.com", "password", None)
        assert result is False

    def test_auth_config(self):
        """Test authentication configuration"""
        config = AuthConfig()

        assert config.secret_key is not None
        assert config.algorithm == "HS256"
        assert isinstance(config.access_token_expire_minutes, int)
        assert config.access_token_expire_minutes > 0

    def test_auth_config_environment_variables(self):
        """Test auth config reads from environment variables"""
        # Set environment variables
        test_secret = "test-secret-key-123"
        test_expiry = "60"

        os.environ["SECRET_KEY"] = test_secret
        os.environ["ACCESS_TOKEN_EXPIRE_MINUTES"] = test_expiry

        try:
            config = AuthConfig()
            assert config.secret_key == test_secret
            assert config.access_token_expire_minutes == 60
        finally:
            # Clean up environment
            os.environ.pop("SECRET_KEY", None)
            os.environ.pop("ACCESS_TOKEN_EXPIRE_MINUTES", None)


class TestDatabaseFunctionality:
    """Test database core functions"""

    def test_create_db_and_tables(self):
        """Test database and table creation"""
        # Create temporary database
        temp_db = tempfile.NamedTemporaryFile(suffix=".db", delete=False)
        temp_db.close()

        try:
            # Create engine with temporary database
            test_engine = create_engine(f"sqlite:///{temp_db.name}", echo=False)

            # Test that we can create tables without errors
            import goldfish_backend.models  # noqa - Import to register models
            SQLModel.metadata.create_all(test_engine)

            # Verify tables exist by creating a session and user
            with Session(test_engine) as session:
                user = User(
                    email="test@example.com",
                    hashed_password="test_hash",
                    full_name="Test User"
                )
                session.add(user)
                session.commit()
                session.refresh(user)

                assert user.id is not None

        finally:
            # Cleanup
            os.unlink(temp_db.name)

    def test_database_engine_exists(self):
        """Test that database engine is properly configured"""

        assert engine is not None
        assert hasattr(engine, 'connect')
        assert hasattr(engine, 'url')

    def test_database_session_management(self):
        """Test database session creation and management"""
        # Create temporary database for testing
        temp_db = tempfile.NamedTemporaryFile(suffix=".db", delete=False)
        temp_db.close()

        try:
            test_engine = create_engine(f"sqlite:///{temp_db.name}", echo=False)

            # Import models to register them
            import goldfish_backend.models  # noqa
            SQLModel.metadata.create_all(test_engine)

            # Test session creation and usage
            with Session(test_engine) as session:
                # Create user
                user = User(
                    email="session_test@example.com",
                    hashed_password="test_hash",
                    full_name="Session Test User"
                )
                session.add(user)
                session.commit()
                session.refresh(user)

                # Verify user was created
                assert user.id is not None
                assert user.email == "session_test@example.com"

            # Test that session is properly closed (new session should work)
            with Session(test_engine) as new_session:
                from sqlmodel import select

                # Query user from new session
                statement = select(User).where(User.email == "session_test@example.com")
                retrieved_user = new_session.exec(statement).first()

                assert retrieved_user is not None
                assert retrieved_user.email == "session_test@example.com"

        finally:
            # Cleanup
            os.unlink(temp_db.name)


class TestCoreIntegration:
    """Test integration between core components"""

    def test_auth_and_database_integration(self, db_session: Session):
        """Test that authentication works with database"""
        # Create user with proper password hashing
        email = "integration_test@example.com"
        password = "TestPassword123!"

        user = User(
            email=email,
            hashed_password=get_password_hash(password),
            full_name="Integration Test User"
        )
        db_session.add(user)
        db_session.commit()
        db_session.refresh(user)

        # Test full authentication flow
        # 1. Verify password hashing worked
        assert verify_password(password, user.hashed_password) is True

        # 2. Test user authentication
        auth_result = authenticate_user(email, password, user)
        assert auth_result == user

        # 3. Create JWT token for authenticated user
        token_data = {"sub": user.email, "user_id": user.id}
        token = create_access_token(token_data)
        assert isinstance(token, str)
        assert len(token) > 0

    def test_password_change_workflow(self, db_session: Session):
        """Test complete password change workflow"""
        # Create user
        email = "password_change_test@example.com"
        old_password = "OldPassword123!"
        new_password = "NewPassword456!"

        user = User(
            email=email,
            hashed_password=get_password_hash(old_password),
            full_name="Password Change Test User"
        )
        db_session.add(user)
        db_session.commit()
        db_session.refresh(user)

        # Verify old password works
        assert authenticate_user(email, old_password, user) == user
        assert authenticate_user(email, new_password, user) is False

        # Change password
        user.hashed_password = get_password_hash(new_password)
        user.updated_at = datetime.utcnow()
        db_session.add(user)
        db_session.commit()
        db_session.refresh(user)

        # Verify new password works and old doesn't
        assert authenticate_user(email, new_password, user) == user
        assert authenticate_user(email, old_password, user) is False

    def test_user_lifecycle_with_auth(self, db_session: Session):
        """Test complete user lifecycle with authentication"""
        # 1. Create user with strong password
        email = "lifecycle_test@example.com"
        password = "LifecycleTest123!"

        # Validate password first
        assert validate_password_strength(password) is True

        user = User(
            email=email,
            hashed_password=get_password_hash(password),
            full_name="Lifecycle Test User",
            is_active=True,
            is_deleted=False
        )
        db_session.add(user)
        db_session.commit()
        db_session.refresh(user)

        # 2. Authenticate active user
        auth_result = authenticate_user(email, password, user)
        assert auth_result == user

        # 3. Deactivate user
        user.is_active = False
        user.updated_at = datetime.utcnow()
        db_session.add(user)
        db_session.commit()

        # Note: Authentication should still work for deactivated users
        # (application logic should check is_active separately)
        auth_result = authenticate_user(email, password, user)
        assert auth_result == user
        assert user.is_active is False

        # 4. Soft delete user
        user.is_deleted = True
        user.updated_at = datetime.utcnow()
        db_session.add(user)
        db_session.commit()

        # Authentication should still work (soft delete doesn't affect auth)
        # but application should check is_deleted flag
        auth_result = authenticate_user(email, password, user)
        assert auth_result == user
        assert user.is_deleted is True


class TestSecurityFeatures:
    """Test security-related functionality"""

    def test_password_hash_uniqueness(self):
        """Test that same password produces different hashes"""
        password = "TestPassword123!"

        hash1 = get_password_hash(password)
        hash2 = get_password_hash(password)

        # Hashes should be different (due to salt)
        assert hash1 != hash2

        # But both should verify correctly
        assert verify_password(password, hash1) is True
        assert verify_password(password, hash2) is True

    def test_token_contains_no_sensitive_data(self):
        """Test that JWT tokens don't contain sensitive information"""
        import base64
        import json

        # Create token with user data
        user_data = {
            "sub": "test@example.com",
            "user_id": 123
        }
        token = create_access_token(user_data)

        # Decode token payload (without verification, just to check contents)
        # JWT format: header.payload.signature
        parts = token.split('.')
        assert len(parts) == 3

        # Decode payload (add padding if needed)
        payload_b64 = parts[1]
        # Add padding if needed
        payload_b64 += '=' * (4 - len(payload_b64) % 4)
        payload_json = base64.b64decode(payload_b64)
        payload_data = json.loads(payload_json)

        # Verify no sensitive data in token
        assert "password" not in payload_data
        assert "hashed_password" not in payload_data
        assert "secret" not in str(payload_data).lower()

        # Should contain expected user data
        assert payload_data["sub"] == "test@example.com"
        assert payload_data["user_id"] == 123
        assert "exp" in payload_data  # Expiration time

    def test_password_validation_edge_cases(self):
        """Test password validation with edge cases"""
        edge_cases = [
            "",                        # Empty
            " " * 10,                 # Only spaces
            "12345678",               # Only numbers
            "abcdefgh",               # Only lowercase
            "ABCDEFGH",               # Only uppercase
            "!@#$%^&*",               # Only special chars
            "aA1!",                   # Too short but has all types
            "a" * 100 + "A1!",        # Very long but valid
        ]

        for password in edge_cases[:-1]:  # All should be invalid except last
            assert validate_password_strength(password) is False, f"Password '{password}' should be invalid"

        # Last one should be valid (very long but has all requirements)
        assert validate_password_strength(edge_cases[-1]) is True



"""
Tests for entity recognition service
"""
import pytest

from goldfish_backend.services.entity_recognition import EntityRecognitionEngine


class TestEntityRecognitionEngine:
    """Test EntityRecognitionEngine functionality"""

    def test_engine_initialization(self):
        """Test that engine initializes without errors"""
        engine = EntityRecognitionEngine()
        assert engine is not None

    def test_process_text_with_people_mentions(self):
        """Test processing text with @people mentions"""
        engine = EntityRecognitionEngine()

        text = "Meeting with @sarah and @john about the project tomorrow"
        result = engine.process_text(text)

        assert "entities" in result
        assert "tasks" in result
        assert "total_entities" in result
        assert "total_tasks" in result

        # Should detect people entities
        people = result["entities"].get("people", [])
        assert len(people) >= 2

        # Check for sarah and john
        names = [person.name.lower() for person in people]
        assert "sarah" in names
        assert "john" in names

    def test_process_text_with_project_hashtags(self):
        """Test processing text with #project hashtags"""
        engine = EntityRecognitionEngine()

        text = "Working on #ai-platform and #blockchain-integration projects"
        result = engine.process_text(text)

        # Should detect project entities
        projects = result["entities"].get("projects", [])
        assert len(projects) >= 2

        # Check project names
        project_names = [proj.name.lower() for proj in projects]
        assert any("ai-platform" in name or "ai platform" in name for name in project_names)
        assert any("blockchain-integration" in name or "blockchain integration" in name for name in project_names)

    def test_process_text_with_todo_tasks(self):
        """Test processing text with TODO tasks"""
        engine = EntityRecognitionEngine()

        text = """
        TODO: Follow up with client about requirements
        Need to schedule meeting with team
        TODO: Review and approve design documents
        """

        result = engine.process_text(text)

        # Should detect tasks
        tasks = result["tasks"]
        assert len(tasks) >= 2

        # Check task content
        task_contents = [task.content.lower() for task in tasks]
        assert any("follow up" in content for content in task_contents)
        assert any("review" in content and "approve" in content for content in task_contents)

    def test_process_text_with_mixed_entities(self):
        """Test processing text with multiple entity types"""
        engine = EntityRecognitionEngine()

        text = """
        TODO: Follow up with @sarah about the #blockchain-integration project.
        Need to discuss the smart contract architecture and @john's feedback.
        Also review the #ai-platform roadmap for Q2.
        """

        result = engine.process_text(text)

        # Should detect multiple entity types
        assert len(result["entities"]["people"]) >= 2
        assert len(result["entities"]["projects"]) >= 2
        assert len(result["tasks"]) >= 1

        # Check confidence scores
        for _entity_type, entities in result["entities"].items():
            for entity in entities:
                assert 0.0 <= entity.confidence <= 1.0
                assert entity.name is not None
                assert entity.context is not None

    def test_process_empty_text(self):
        """Test processing empty or minimal text"""
        engine = EntityRecognitionEngine()

        result = engine.process_text("")

        assert result["total_entities"] == 0
        assert result["total_tasks"] == 0
        assert len(result["entities"]["people"]) == 0
        assert len(result["entities"]["projects"]) == 0
        assert len(result["entities"]["topics"]) == 0
        assert len(result["tasks"]) == 0

    def test_confidence_scores_reasonable(self):
        """Test that confidence scores are reasonable"""
        engine = EntityRecognitionEngine()

        # High confidence cases - test @mentions and #hashtags specifically
        high_confidence_text = "Meeting with @sarah about #blockchain-platform"
        result = engine.process_text(high_confidence_text)

        # Check that @mentions have high confidence
        people = result["entities"]["people"]
        assert len(people) > 0
        mention_people = [p for p in people if p.original_text.startswith("@")]
        assert all(p.confidence >= 0.8 for p in mention_people)

        # Check that #hashtags have high confidence
        projects = result["entities"]["projects"]
        assert len(projects) > 0
        hashtag_projects = [p for p in projects if p.original_text.startswith("#")]
        assert all(p.confidence >= 0.8 for p in hashtag_projects)

    def test_context_preservation(self):
        """Test that entity context is preserved correctly"""
        engine = EntityRecognitionEngine()

        text = "Scheduled a call with @alice to discuss the quarterly #budget-review"
        result = engine.process_text(text)

        # Check that context contains relevant surrounding text
        for _entity_type, entities in result["entities"].items():
            for entity in entities:
                assert text in entity.context or entity.context in text

    def test_duplicate_entity_handling(self):
        """Test handling of duplicate entities in text"""
        engine = EntityRecognitionEngine()

        text = "@john mentioned that @john will handle the @john's project"
        result = engine.process_text(text)

        # Should not create duplicate entities
        people = result["entities"]["people"]
        john_entities = [p for p in people if "john" in p.name.lower()]

        # Should consolidate or handle duplicates appropriately
        assert len(john_entities) <= 3  # At most one per occurrence

    def test_case_insensitive_matching(self):
        """Test that entity recognition is case insensitive"""
        engine = EntityRecognitionEngine()

        text1 = "Meeting with @Sarah about #PROJECT-ALPHA"
        text2 = "meeting with @sarah about #project-alpha"

        result1 = engine.process_text(text1)
        result2 = engine.process_text(text2)

        # Should detect entities regardless of case
        assert len(result1["entities"]["people"]) > 0
        assert len(result1["entities"]["projects"]) > 0
        assert len(result2["entities"]["people"]) > 0
        assert len(result2["entities"]["projects"]) > 0

    def test_special_characters_in_entities(self):
        """Test handling of special characters in entity names"""
        engine = EntityRecognitionEngine()

        text = "Working with @marie-claire on #ai-ml-project and #web3.0-platform"
        result = engine.process_text(text)

        # Should handle hyphens, dots, etc. in entity names
        people = result["entities"]["people"]
        projects = result["entities"]["projects"]

        assert len(people) > 0
        assert len(projects) > 0

        # Check that special characters are preserved or handled correctly
        person_names = [p.name for p in people]
        project_names = [p.name for p in projects]

        assert any("marie" in name.lower() for name in person_names)
        assert any("ai" in name.lower() or "ml" in name.lower() for name in project_names)

    @pytest.mark.slow
    def test_large_text_processing(self):
        """Test processing large amounts of text"""
        engine = EntityRecognitionEngine()

        # Create large text with multiple entities
        large_text = """
        Project Status Update - Week 45

        This week we made significant progress on the #ai-platform initiative.
        @sarah completed the initial data pipeline architecture, while @john
        worked on the machine learning model integration.

        TODO: Review the proposed changes with @alice before the next sprint
        TODO: Schedule demo session for stakeholders
        TODO: Update documentation for the #blockchain-integration module

        The #mobile-app development is also progressing well. @bob has finished
        the authentication flow, and @carol is working on the user interface.

        Upcoming tasks:
        - Follow up with @david about the API specifications
        - Need to discuss #database-optimization strategies
        - TODO: Plan the #deployment-pipeline for staging environment

        Overall, the team is on track to meet the Q4 objectives for both
        #ai-platform and #mobile-app projects.
        """ * 3  # Make it larger

        result = engine.process_text(large_text)

        # Should handle large text without errors
        assert result is not None
        assert result["total_entities"] > 0
        assert result["total_tasks"] > 0

        # Should detect multiple entities
        assert len(result["entities"]["people"]) >= 5
        assert len(result["entities"]["projects"]) >= 3
        assert len(result["tasks"]) >= 3


class TestEntityRecognitionPatterns:
    """Test specific pattern recognition"""

    def test_email_mention_pattern(self):
        """Test that email addresses don't get confused with @mentions"""
        engine = EntityRecognitionEngine()

        text = "Send email to user@example.com and also ping @alice"
        result = engine.process_text(text)

        people = result["entities"]["people"]
        # Should only detect @alice, not the email
        assert len(people) == 1
        assert "alice" in people[0].name.lower()

    def test_hashtag_vs_hash_symbol(self):
        """Test hashtag detection vs regular hash symbols"""
        engine = EntityRecognitionEngine()

        text = "Password: #secure123 vs project #web-development"
        result = engine.process_text(text)

        projects = result["entities"]["projects"]
        # Should detect project hashtag but not password hash
        assert len(projects) >= 1
        project_names = [p.name.lower() for p in projects]
        assert any("web" in name for name in project_names)

    def test_multiple_todo_formats(self):
        """Test recognition of different TODO formats"""
        engine = EntityRecognitionEngine()

        text = """
        TODO: First task
        - TODO: Second task
        * TODO: Third task
        Need to complete fourth task
        Should finish fifth task
        """

        result = engine.process_text(text)

        tasks = result["tasks"]
        # Should detect multiple TODO formats
        assert len(tasks) >= 3

        task_contents = [task.content.lower() for task in tasks]
        assert any("first" in content for content in task_contents)
        assert any("second" in content for content in task_contents)
        assert any("third" in content for content in task_contents)



"""
User model for authentication and user management
"""
from datetime import datetime

from sqlmodel import Column, Field, SQLModel, String


class User(SQLModel, table=True):
    """User table for authentication and profile information"""

    __tablename__ = "users"

    id: int | None = Field(default=None, primary_key=True)
    email: str = Field(sa_column=Column(String, unique=True, index=True))
    hashed_password: str
    full_name: str
    bio: str | None = None

    # Timestamps
    created_at: datetime = Field(default_factory=datetime.utcnow)
    updated_at: datetime | None = None

    # Status flags
    is_active: bool = Field(default=True)
    is_deleted: bool = Field(default=False)

    class Config:
        """SQLModel configuration"""
        from_attributes = True



"""
Person model for entity linking and human verification
"""
from datetime import datetime

from sqlmodel import JSON, Column, Field, SQLModel, String, Text


class Person(SQLModel, table=True):
    """Person entity table for people recognition and linking"""

    __tablename__ = "people"

    id: int | None = Field(default=None, primary_key=True)
    user_id: int = Field(foreign_key="users.id", index=True)
    name: str = Field(sa_column=Column(String(255)))
    aliases: list[str] = Field(default_factory=list, sa_column=Column(JSON))
    importance_score: float = Field(default=1.0, ge=0.0, le=10.0)

    # Contact information
    bio: str | None = Field(default=None, sa_column=Column(Text))
    email: str | None = Field(default=None, sa_column=Column(String(255)))
    phone: str | None = Field(default=None, sa_column=Column(String(50)))

    # Timestamps
    created_at: datetime = Field(default_factory=datetime.utcnow)
    updated_at: datetime | None = None

    # Status flags
    is_deleted: bool = Field(default=False)

    class Config:
        """SQLModel configuration"""
        from_attributes = True



"""
Entity learning model for improving AI accuracy from user feedback
"""
from datetime import datetime
from typing import Any

from sqlmodel import JSON, Column, Field, SQLModel, String, Text


class EntityLearning(SQLModel, table=True):
    """Entity learning table for recording user confirmations to improve AI"""

    __tablename__ = "entity_learning"

    id: int | None = Field(default=None, primary_key=True)
    user_id: int = Field(foreign_key="users.id", index=True)
    original_text: str = Field(sa_column=Column(String(500)))
    linked_entity_id: int
    entity_type: str = Field(sa_column=Column(String(20)))  # person, project, topic
    context: str = Field(sa_column=Column(Text))
    source_file_path: str = Field(sa_column=Column(String(1000)))
    confidence: float = Field(default=1.0, ge=0.0, le=1.0)

    # Timestamps
    created_at: datetime = Field(default_factory=datetime.utcnow)

    # Additional learning context
    learning_metadata: dict[str, Any] | None = Field(default=None, sa_column=Column(JSON))

    class Config:
        """SQLModel configuration"""
        from_attributes = True



"""
Task model for actionable items extracted from notes
"""
from datetime import datetime

from sqlmodel import Column, Field, SQLModel, String, Text


class Task(SQLModel, table=True):
    """Task table for actionable items with AI-calculated priorities"""

    __tablename__ = "tasks"

    id: int | None = Field(default=None, primary_key=True)
    user_id: int = Field(foreign_key="users.id", index=True)
    note_id: int = Field(foreign_key="notes.id")
    content: str = Field(sa_column=Column(Text))
    status: str = Field(default="pending", sa_column=Column(String(20)))  # pending, in_progress, completed, cancelled
    priority_score: float = Field(default=0.0, ge=0.0, le=10.0)
    position: int = Field(default=0)  # Position in kanban column
    due_date: datetime | None = None

    # Timestamps
    created_at: datetime = Field(default_factory=datetime.utcnow)
    updated_at: datetime | None = None
    completed_at: datetime | None = None

    # Status flags
    is_deleted: bool = Field(default=False)

    class Config:
        """SQLModel configuration"""
        from_attributes = True



"""
Suggested entity model for human-in-the-loop verification
"""
from datetime import datetime
from typing import Any

from sqlmodel import JSON, Column, Field, SQLModel, String, Text


class SuggestedEntity(SQLModel, table=True):
    """Suggested entity table for AI suggestions awaiting human verification"""

    __tablename__ = "suggested_entities"

    id: int | None = Field(default=None, primary_key=True)
    user_id: int = Field(foreign_key="users.id", index=True)
    note_id: int = Field(foreign_key="notes.id")
    entity_type: str = Field(sa_column=Column(String(20)))  # person, project, topic
    name: str = Field(sa_column=Column(String(255)))
    context: str = Field(sa_column=Column(Text))
    confidence: float = Field(ge=0.0, le=1.0)
    status: str = Field(default="pending", sa_column=Column(String(20)))  # pending, confirmed, rejected

    # Timestamps
    created_at: datetime = Field(default_factory=datetime.utcnow)
    updated_at: datetime | None = None

    # Additional AI processing metadata
    ai_metadata: dict[str, Any] | None = Field(default=None, sa_column=Column(JSON))

    class Config:
        """SQLModel configuration"""
        from_attributes = True



"""
Tests for goldfish-backend models
"""
from datetime import datetime

import pytest
from faker import Faker
from sqlmodel import Session, select

from goldfish_backend.models.note import Note
from goldfish_backend.models.person import Person
from goldfish_backend.models.project import Project
from goldfish_backend.models.suggested_entity import SuggestedEntity
from goldfish_backend.models.user import User

fake = Faker()


class TestUserModel:
    """Test User model functionality"""

    def test_create_user_basic(self, db_session: Session):
        """Test basic user creation"""
        email = fake.email()
        user = User(
            email=email,
            hashed_password="hashed_password_123",
            full_name=fake.name(),
            bio=fake.text(max_nb_chars=200)
        )

        db_session.add(user)
        db_session.commit()
        db_session.refresh(user)

        assert user.id is not None
        assert user.email == email
        assert user.is_active is True
        assert user.is_deleted is False
        assert isinstance(user.created_at, datetime)

    def test_user_email_unique(self, db_session: Session):
        """Test that user emails are unique"""
        email = fake.email()

        # Create first user
        user1 = User(
            email=email,
            hashed_password="hash1",
            full_name="User One"
        )
        db_session.add(user1)
        db_session.commit()

        # Try to create second user with same email
        user2 = User(
            email=email,
            hashed_password="hash2",
            full_name="User Two"
        )
        db_session.add(user2)

        with pytest.raises((Exception, ValueError)):  # Should raise integrity error
            db_session.commit()

        # Rollback the failed transaction
        db_session.rollback()

    def test_user_defaults(self, db_session: Session):
        """Test user model defaults"""
        user = User(
            email=fake.email(),
            hashed_password="test_hash",
            full_name=fake.name()
        )

        assert user.bio is None
        assert user.updated_at is None
        assert user.is_active is True
        assert user.is_deleted is False
        assert user.created_at is not None


class TestPersonModel:
    """Test Person model functionality"""

    def test_create_person(self, db_session: Session):
        """Test basic person creation"""
        # Create a user first
        user = User(
            email=fake.email(),
            hashed_password="test_hash",
            full_name=fake.name()
        )
        db_session.add(user)
        db_session.commit()
        db_session.refresh(user)

        # Create person
        person_name = fake.name()
        person = Person(
            user_id=user.id,
            name=person_name,
            aliases=["alias1", "alias2"],
            importance_score=7.5,
            bio=fake.text(),
            email=fake.email()
        )

        db_session.add(person)
        db_session.commit()
        db_session.refresh(person)

        assert person.id is not None
        assert person.user_id == user.id
        assert person.name == person_name
        assert person.aliases == ["alias1", "alias2"]
        assert person.importance_score == 7.5
        assert person.is_deleted is False
        assert isinstance(person.created_at, datetime)

    def test_person_defaults(self, db_session: Session):
        """Test person model defaults"""
        # Create user first
        user = User(
            email=fake.email(),
            hashed_password="test_hash",
            full_name=fake.name()
        )
        db_session.add(user)
        db_session.commit()
        db_session.refresh(user)

        # Create person with minimal data
        person = Person(
            user_id=user.id,
            name=fake.name()
        )

        assert person.aliases == []
        assert person.importance_score == 1.0
        assert person.bio is None
        assert person.email is None
        assert person.phone is None
        assert person.is_deleted is False

    def test_person_importance_score_validation(self, db_session: Session):
        """Test importance score validation (0-10 range)"""
        # Create user first
        user = User(
            email=fake.email(),
            hashed_password="test_hash",
            full_name=fake.name()
        )
        db_session.add(user)
        db_session.commit()
        db_session.refresh(user)

        # Test valid scores
        for score in [0.0, 5.5, 10.0]:
            person = Person(
                user_id=user.id,
                name=fake.name(),
                importance_score=score
            )
            db_session.add(person)
            db_session.commit()
            assert person.importance_score == score
            db_session.delete(person)
            db_session.commit()


class TestProjectModel:
    """Test Project model functionality"""

    def test_create_project(self, db_session: Session):
        """Test basic project creation"""
        # Create user first
        user = User(
            email=fake.email(),
            hashed_password="test_hash",
            full_name=fake.name()
        )
        db_session.add(user)
        db_session.commit()
        db_session.refresh(user)

        # Create project
        project_name = fake.company()
        project = Project(
            user_id=user.id,
            name=project_name,
            description=fake.text(),
            status="active",
            priority_score=8.0
        )

        db_session.add(project)
        db_session.commit()
        db_session.refresh(project)

        assert project.id is not None
        assert project.user_id == user.id
        assert project.name == project_name
        assert project.status == "active"
        assert project.priority_score == 8.0
        assert project.is_deleted is False


class TestNoteModel:
    """Test Note model functionality"""

    def test_create_note(self, db_session: Session):
        """Test basic note creation"""
        # Create user first
        user = User(
            email=fake.email(),
            hashed_password="test_hash",
            full_name=fake.name()
        )
        db_session.add(user)
        db_session.commit()
        db_session.refresh(user)

        # Create source file first
        from goldfish_backend.models.source_file import SourceFile
        source_file = SourceFile(
            user_id=user.id,
            file_path="/test/path.md",
            relative_path="path.md",
            file_hash="test_file_hash",
            file_size=1000,
            last_modified=datetime.utcnow()
        )
        db_session.add(source_file)
        db_session.commit()
        db_session.refresh(source_file)

        # Create note
        content = fake.text()
        note = Note(
            user_id=user.id,
            source_file_id=source_file.id,
            content=content,
            content_hash="test_hash",
            snapshot_at=datetime.utcnow(),
            processing_metadata={"source": "test"}
        )

        db_session.add(note)
        db_session.commit()
        db_session.refresh(note)

        assert note.id is not None
        assert note.user_id == user.id
        assert note.source_file_id == source_file.id
        assert note.content == content
        assert note.processing_metadata == {"source": "test"}
        assert note.is_deleted is False


class TestSuggestedEntityModel:
    """Test SuggestedEntity model functionality"""

    def test_create_suggested_entity(self, db_session: Session):
        """Test basic suggested entity creation"""
        # Create user first
        user = User(
            email=fake.email(),
            hashed_password="test_hash",
            full_name=fake.name()
        )
        db_session.add(user)
        db_session.commit()
        db_session.refresh(user)

        # Create source file and note first
        from goldfish_backend.models.source_file import SourceFile
        source_file = SourceFile(
            user_id=user.id,
            file_path="/test/path.md",
            relative_path="path.md",
            file_hash="test_file_hash",
            file_size=1000,
            last_modified=datetime.utcnow()
        )
        db_session.add(source_file)
        db_session.commit()
        db_session.refresh(source_file)

        note = Note(
            user_id=user.id,
            source_file_id=source_file.id,
            content="Meeting with @john tomorrow",
            content_hash="test_hash",
            snapshot_at=datetime.utcnow()
        )
        db_session.add(note)
        db_session.commit()
        db_session.refresh(note)

        # Create suggested entity
        suggestion = SuggestedEntity(
            user_id=user.id,
            note_id=note.id,
            entity_type="person",
            name="John Doe",
            confidence=0.95,
            context="Meeting with @john tomorrow",
            status="pending",
            ai_metadata={"pattern": "@mention"}
        )

        db_session.add(suggestion)
        db_session.commit()
        db_session.refresh(suggestion)

        assert suggestion.id is not None
        assert suggestion.user_id == user.id
        assert suggestion.note_id == note.id
        assert suggestion.entity_type == "person"
        assert suggestion.name == "John Doe"
        assert suggestion.confidence == 0.95
        assert suggestion.status == "pending"
        assert suggestion.ai_metadata == {"pattern": "@mention"}

    def test_suggested_entity_confidence_validation(self, db_session: Session):
        """Test confidence score validation (0-1 range)"""
        # Create user first
        user = User(
            email=fake.email(),
            hashed_password="test_hash",
            full_name=fake.name()
        )
        db_session.add(user)
        db_session.commit()
        db_session.refresh(user)

        # Create source file and note first
        from goldfish_backend.models.source_file import SourceFile
        source_file = SourceFile(
            user_id=user.id,
            file_path="/test/path.md",
            relative_path="path.md",
            file_hash="test_file_hash",
            file_size=1000,
            last_modified=datetime.utcnow()
        )
        db_session.add(source_file)
        db_session.commit()
        db_session.refresh(source_file)

        note = Note(
            user_id=user.id,
            source_file_id=source_file.id,
            content="test context",
            content_hash="test_hash",
            snapshot_at=datetime.utcnow()
        )
        db_session.add(note)
        db_session.commit()
        db_session.refresh(note)

        # Test valid confidence scores
        for confidence in [0.0, 0.5, 1.0]:
            suggestion = SuggestedEntity(
                user_id=user.id,
                note_id=note.id,
                entity_type="person",
                name="Test",
                confidence=confidence,
                context="test context"
            )
            db_session.add(suggestion)
            db_session.commit()
            assert suggestion.confidence == confidence
            db_session.delete(suggestion)
            db_session.commit()


class TestModelRelationships:
    """Test relationships between models"""

    def test_user_can_have_multiple_entities(self, db_session: Session):
        """Test that a user can have multiple people, projects, etc."""
        # Create user
        user = User(
            email=fake.email(),
            hashed_password="test_hash",
            full_name=fake.name()
        )
        db_session.add(user)
        db_session.commit()
        db_session.refresh(user)

        # Create multiple entities for the user
        person1 = Person(user_id=user.id, name="Person 1")
        person2 = Person(user_id=user.id, name="Person 2")
        project1 = Project(user_id=user.id, name="Project 1")
        project2 = Project(user_id=user.id, name="Project 2")

        db_session.add_all([person1, person2, project1, project2])
        db_session.commit()

        # Verify user has multiple entities
        people = db_session.exec(
            select(Person).where(Person.user_id == user.id)
        ).all()
        projects = db_session.exec(
            select(Project).where(Project.user_id == user.id)
        ).all()

        assert len(people) == 2
        assert len(projects) == 2
        assert all(p.user_id == user.id for p in people)
        assert all(p.user_id == user.id for p in projects)

    def test_soft_delete_filtering(self, db_session: Session):
        """Test that soft-deleted entities are properly filtered"""
        # Create user and person
        user = User(
            email=fake.email(),
            hashed_password="test_hash",
            full_name=fake.name()
        )
        db_session.add(user)
        db_session.commit()
        db_session.refresh(user)

        person = Person(user_id=user.id, name="Test Person")
        db_session.add(person)
        db_session.commit()
        db_session.refresh(person)

        # Verify person exists
        active_people = db_session.exec(
            select(Person).where(Person.user_id == user.id, Person.is_deleted == False)
        ).all()
        assert len(active_people) == 1

        # Soft delete person
        person.is_deleted = True
        db_session.add(person)
        db_session.commit()

        # Verify person is filtered out
        active_people = db_session.exec(
            select(Person).where(Person.user_id == user.id, Person.is_deleted == False)
        ).all()
        assert len(active_people) == 0

        # But still exists when not filtering
        all_people = db_session.exec(
            select(Person).where(Person.user_id == user.id)
        ).all()
        assert len(all_people) == 1
